<?xml version="1.0"?>
<xsl:stylesheet 
  version='1.0' 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exslt="http://exslt.org/common"
  xmlns:math="http://exslt.org/math"
  extension-element-prefixes="exslt math">

  <!-- Convert sections of an EtreCheck report into HTML 
       representations. -->
  <xsl:output method="html" indent="yes" encoding="UTF-8"/> 
 
  <!-- Set the language. -->
  <xsl:param name="language">en</xsl:param>

  <!-- Set the section. -->
  <xsl:param name="section"/>

  <!-- Set the port. -->
  <xsl:param name="port"/>

  <!-- Language-specific data. -->
  <xsl:variable 
    name="strings" 
    select="document(concat('UI_', $language, '.xml'))/strings"/>
  
  <!-- Language-specific issues. -->
  <xsl:variable 
    name="issues" 
    select="document(concat('Issues_', $language, '.xml'))/issues"/>

  <!-- Get rid of these. -->
  <xsl:variable name="performance" select="$strings/performance_rating"/>

  <xsl:variable name="statusattributes" select="$strings/status"/>
  
  <!-- This is for the user interface. -->
  <xsl:template match="/etrecheck">
  
    <xsl:text disable-output-escaping='yes'>&lt;!DOCTYPE html&gt;
</xsl:text>
    <html>
      <xsl:attribute name="lang">
        <xsl:value-of select="$language"/>
      </xsl:attribute>
      
      <head>

        <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>

        <title>EtreCheck Report</title>

        <link href="/css/etrecheck.css" type="text/css" rel="stylesheet"/>

        <!-- Frameworks stuff. -->
        <script src="/script/jquery.js"></script>

        <script src="/script/etrecheck.js"></script>

      </head>
      
      <body>
      
        <div class="scrollfix">
        
          <xsl:choose>
          
            <xsl:when test="$section = 'start'">
              <xsl:apply-templates select="header" mode="top"/>
            </xsl:when>
            
            <xsl:when test="$section = 'major'">
              <xsl:call-template name="major"/>
            </xsl:when>

            <xsl:when test="$section = 'minor'">
              <xsl:call-template name="minor"/>
            </xsl:when>

            <xsl:when test="$section = 'hardware'">
              <xsl:apply-templates select="hardware" mode="top"/>
              <xsl:apply-templates select="video" mode="top"/>
            </xsl:when>

            <xsl:when test="$section = 'system'">
              <xsl:apply-templates select="systemsoftware" mode="top"/>
              <xsl:apply-templates select="configurationfiles" mode="top"/>
              <xsl:apply-templates select="startupitems" mode="top"/>
              <xsl:apply-templates select="fonts" mode="top"/>
              <xsl:apply-templates select="vm" mode="top"/>
              <xsl:apply-templates select="diagnostics" mode="top"/>
            </xsl:when>

            <!-- Storage is special because it pulls from a number of
                 different interfaces and sources and renames them. -->
            <xsl:when test="$section = 'storage'">
              <xsl:apply-templates select="disk" mode="top"/>
              <xsl:call-template name="diskimage"/>
              <xsl:apply-templates select="virtualvolume" mode="top"/>
              <xsl:apply-templates select="timemachine" mode="top"/>
            </xsl:when>

            <xsl:when test="$section = 'network'">
              <xsl:apply-templates select="network" mode="top"/>
              <xsl:apply-templates select="networkusage" mode="top"/>
            </xsl:when>

            <xsl:when test="$section = 'security'">
              <xsl:apply-templates select="security" mode="top"/>
              <xsl:apply-templates select="adware" mode="top"/>
              <xsl:apply-templates select="unsigned" mode="top"/>
            </xsl:when>

            <xsl:when test="$section = 'software'">
              <xsl:apply-templates select="kernelextensions" mode="top"/>
              <xsl:apply-templates select="systemlaunchagents" mode="top"/>
              <xsl:apply-templates select="systemlaunchdaemons" mode="top"/>
              <xsl:apply-templates select="launchagents" mode="top"/>
              <xsl:apply-templates select="launchdaemons" mode="top"/>
              <xsl:apply-templates select="userlaunchagents" mode="top"/>
              <xsl:apply-templates select="loginitems" mode="top"/>
              <xsl:apply-templates select="internetplugins" mode="top"/>
              <xsl:apply-templates select="userinternetplugins" mode="top"/>
              <xsl:apply-templates select="audioplugins" mode="top"/>
              <xsl:apply-templates select="useraudioplugins" mode="top"/>
              <xsl:apply-templates select="itunesplugins" mode="top"/>
              <xsl:apply-templates select="useritunesplugins" mode="top"/>
              <xsl:apply-templates select="safariextensions" mode="top"/>
              <xsl:apply-templates select="preferencepanes" mode="top"/>
              <xsl:apply-templates select="install" mode="top"/>
              <xsl:apply-templates select="cleanup" mode="top"/>
            </xsl:when>

            <xsl:when test="$section = 'performance'">
              <xsl:call-template name="performance"/>
              <xsl:apply-templates select="cpu" mode="top"/>
              <xsl:apply-templates select="memory" mode="top"/>
              <xsl:apply-templates select="energy" mode="top"/>
            </xsl:when>

            <xsl:otherwise>
            </xsl:otherwise>

          </xsl:choose>
        
        </div>
  
      </body>

    </html>

  </xsl:template>

  <!-- Header section. -->
  <xsl:template match="header" mode="top">
  
    <!-- These are not divided in the original XML. -->
    
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">header</xsl:with-param>
      <xsl:with-param name="contents">
        <xsl:apply-templates select="."/>
      </xsl:with-param>
    </xsl:call-template>
  
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">instructions</xsl:with-param>
      <xsl:with-param name="contents">
        <xsl:copy-of select="$strings/etrecheck/instructions/node()"/>
      </xsl:with-param>
    </xsl:call-template>

  </xsl:template>
  
  <!-- Header section. -->
  <xsl:template match="header">
  
    <table>
      <tr>
        <td rowspan="3">
          <img src="/image/etrecheck.png">
            <xsl:attribute name="alt">
              <xsl:value-of select="$strings/etrecheck/logo"/>
            </xsl:attribute>
          </img>
        </td>
        <th><xsl:value-of select="$strings/etrecheck/version"/></th>
        <td>
          <xsl:value-of select="version"/>
          <xsl:text> (</xsl:text>
          <xsl:value-of select="build"/>
          <xsl:text>)</xsl:text>
        </td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/etrecheck/date"/></th>
        <td><xsl:value-of select="date"/></td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/etrecheck/problem"/></th>
        <td><xsl:value-of select="problem"/></td>
      </tr>
      
      <!-- Problem description is optional. -->
      <xsl:if test="problemdescription">
        <tr>
          <td/>
          <th class="problemdescription">
            <xsl:value-of select="$strings/etrecheck/problem_description"/>
          </th>
          <td class="problemdescription">
            <xsl:value-of select="problemdescription"/>
          </td>
        </tr>
      </xsl:if>
      
    </table>

  </xsl:template>

  <xsl:include href="Issues.xsl" />

  <!-- Major issues. -->
  <xsl:template name="major">
  
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">major</xsl:with-param>
      <xsl:with-param name="contents">
        <xsl:variable name="content">
          <xsl:call-template name="majorissues"/>
        </xsl:variable>

        <table class="tablelist">
          <xsl:copy-of select="$content"/>
          
          <xsl:if test="string-length($content) = 0">
            <xsl:call-template name="nomajorissues"/>
          </xsl:if>
        </table>
        
        <p class="description">
    
          <xsl:if test="string-length($content) &gt; 0">
            <xsl:copy-of select="$issues/major/description/node()"/>
            
            <xsl:copy-of select="$issues/major/problemsfound/node()"/>
            <xsl:copy-of select="$issues/major/problemsfound_fix/node()"/>
          </xsl:if>
        </p>
        
      </xsl:with-param>
    </xsl:call-template>
    
  </xsl:template>
    
  <!-- No Major issues. -->
  <xsl:template name="nomajorissues">
  
    <tr class="horizontalborder">
      <td class="noissues">
      
        <xsl:choose>
          
          <xsl:when test="/etrecheck/header/problem = 'No problem - just checking'">
            <xsl:copy-of select="$issues/major/noproblem_justchecking/node()"/>
          </xsl:when>
          
          <xsl:otherwise>
          
            <xsl:copy-of select="$issues/major/noproblem/node()"/>
          
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>jump("report")</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$issues/major/review_title"/>
              </xsl:attribute>
              <xsl:value-of select="$issues/major/review"/>
            </button>
          
          </xsl:otherwise>
          
        </xsl:choose>
        
      </td>
    </tr>

  </xsl:template>
  
  <!-- Minor issues. -->
  <xsl:template name="minor">
  
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">minor</xsl:with-param>
      <xsl:with-param name="contents">
        <xsl:variable name="content">
          <xsl:call-template name="minorissues"/>
        </xsl:variable>

        <table class="tablelist">
          <xsl:copy-of select="$content"/>
          
          <xsl:if test="string-length($content) = 0">
            <xsl:call-template name="nominorissues"/>
          </xsl:if>
        </table>
        
        <p class="description">
          <xsl:if test="string-length($content) &gt; 0">
            <xsl:copy-of select="$issues/minor/description/node()"/>
  
            <xsl:copy-of select="$issues/minor/problemsfound/node()"/>
            <xsl:copy-of select="$issues/minor/problemsfound_fix/node()"/>
          </xsl:if>
        </p>

      </xsl:with-param>
    </xsl:call-template>

  </xsl:template>

  <!-- No Minor issues. -->
  <xsl:template name="nominorissues">
  
    <tr class="horizontalborder">
      <td class="noissues">
        <xsl:choose>
        
          <xsl:when test="/etrecheck/header/problem = 'No problem - just checking'">
            <xsl:copy-of select="$issues/minor/noproblem_justchecking/node()"/>
          </xsl:when>
          
          <xsl:otherwise>
          
            <xsl:copy-of select="$issues/minor/noproblem/node()"/>
        
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>jump("report")</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$issues/minor/review_title"/>
              </xsl:attribute>
              <xsl:value-of select="$issues/minor/review"/>
            </button>
          
          </xsl:otherwise>
          
        </xsl:choose>
      </td>
    </tr>

  </xsl:template>
  
  <!-- Emit an issue. -->
  <xsl:template name="issue">
    <xsl:param name="type"/>
    <xsl:param name="name"/>
    
    <xsl:variable 
      name="issuenodes" select="$issues/*[local-name() = $type]"/>
    
    <xsl:variable 
      name="issue" select="$issuenodes/*[local-name() = $name]"/>
    
    <tr class="horizontalborder">
      <td class="header">
        <xsl:value-of select="$issue/title"/>
      </td>
      <td>
        <xsl:value-of select="$issue/description"/>
      </td>
      <td>
        <button>
          <xsl:attribute name="onclick">
            <xsl:value-of 
              select="concat('jump(&quot;',$issue/section,'&quot;,&quot;',$issue/issue,'&quot;)')"/>
          </xsl:attribute>
          <xsl:attribute name="title">
            <xsl:value-of select="$issuenodes/review_title"/>
          </xsl:attribute>
          <xsl:value-of select="$issuenodes/review"/>
        </button>
      </td>
    </tr>
    
  </xsl:template>
  
  <!-- Hardware information. -->
  <xsl:template match="hardware" mode="top">
  
    <!-- These are not divided in the original XML. -->
    
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">machine</xsl:with-param>
      <xsl:with-param name="contents">
        <xsl:call-template name="machine"/>
      </xsl:with-param>
    </xsl:call-template>
      
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">processor</xsl:with-param>
      <xsl:with-param name="contents">
        <xsl:call-template name="processor"/>
      </xsl:with-param>
    </xsl:call-template>

    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">ram</xsl:with-param>
      <xsl:with-param name="contents">
        <xsl:call-template name="ram"/>
      </xsl:with-param>
    </xsl:call-template>

    <xsl:if test="batteryhealth and batterycyclecount and batterypercent">
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">battery</xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:call-template name="battery"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:if>
    
  </xsl:template>
 
  <!-- Machine information. -->
  <xsl:template name="machine">
  
    <table>
      <tr>
        <td rowspan="4">
          <img>
            <xsl:attribute name="src"> 
              <xsl:value-of 
                select="concat('http://localhost:', $port, '/image/computer.png')"/>
            </xsl:attribute>
            <xsl:attribute name="alt">
              <xsl:value-of select="$strings/machine/icon"/>
            </xsl:attribute>
          </img>
        </td>
        
        <!-- Add some blank space to accomodate the header. -->
        <td>
          <xsl:text>&#160;</xsl:text>
          <br/>
          <xsl:text>&#160;</xsl:text>
          <br/>
          <xsl:text>&#160;</xsl:text>
          <br/>
        </td>
        <td>
          <xsl:text>&#160;</xsl:text>
          <br/>
          <xsl:text>&#160;</xsl:text>
          <br/>
          <xsl:text>&#160;</xsl:text>
          <br/>
        </td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/machine/name"/></th>
        <td><xsl:value-of select="marketingname"/></td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/machine/model"/></th>
        <td>
          <xsl:value-of select="model"/>
          
          <xsl:call-template name="identify">
            <xsl:with-param name="modeltype" select="modeltype"/>
          </xsl:call-template>
          
        </td>
      </tr>
      <tr>
        <td colspan="2">
          <fieldset class="appleresources">
            <legend>
              <xsl:value-of select="$strings/machine/support_resources"/>
            </legend>
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of 
                  select="concat('GET(&quot;/link/',technicalspecificationsurl,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/machine/technical_specifications_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/machine/technical_specifications"/>
            </button>
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of 
                  select="concat('GET(&quot;/link/',userguideurl,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/machine/user_guide_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/machine/user_guide"/>
            </button>
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of 
                  select="concat('GET(&quot;/link/',warrantyandserviceurl,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/machine/warranty_service_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/machine/warranty_service"/>
            </button>
            
            <xsl:choose>
              <xsl:when test="vintage = 'vintage'">
                <br/>
                <br/>
                <xsl:copy-of select="$strings/machine/vintage/node()"/>
                <button class="vintage">
                  <xsl:attribute name="onclick">
                    <xsl:text>GET("/link/https://support.apple.com/HT201624");</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/machine/vintage_button_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/machine/vintage_button"/>
                </button>
              </xsl:when>
              
              <xsl:when test="vintage = 'obsolete'">
                <br/>
                <br/>
                <xsl:copy-of select="$strings/machine/obsolete/node()"/>
                <button class="vintage">
                  <xsl:attribute name="onclick">
                    <xsl:text>GET("/link/https://support.apple.com/HT201624");</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/machine/obsolete_button_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/machine/obsolete_button"/>
                </button>
              </xsl:when>

              <xsl:otherwise/>
              
            </xsl:choose>
            
          </fieldset>
        </td>
      </tr>
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/machine/description/node()"/>
      
      <xsl:if test="vintage = 'vintage' or vintage = 'obsolete'">
        <xsl:copy-of select="$strings/machine/description_vintage/node()"/>
      </xsl:if>
    </p>
    
  </xsl:template>

  <!-- Identification button. -->
  <xsl:template name="identify">
    <xsl:param name="modeltype"/>
    
    <xsl:variable name="url">
      <xsl:choose>
        <xsl:when test="$modeltype = 'MacPro'">
          <xsl:text>https://support.apple.com/HT202888</xsl:text>
        </xsl:when>
        <xsl:when test="$modeltype = 'Macmini'">
          <xsl:text>https://support.apple.com/HT201894</xsl:text>
        </xsl:when>
        <xsl:when test="$modeltype = 'iMac'">
          <xsl:text>https://support.apple.com/HT201634</xsl:text>
        </xsl:when>
        <xsl:when test="$modeltype = 'MacBookPro'">
          <xsl:text>https://support.apple.com/HT201300</xsl:text>
        </xsl:when>
        <xsl:when test="$modeltype = 'MacBook'">
          <xsl:text>https://support.apple.com/HT201608</xsl:text>
        </xsl:when>
        <xsl:when test="$modeltype = 'MacBookAir'">
          <xsl:text>https://support.apple.com/HT201862</xsl:text>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:if test="string-length($url) &gt; 0">
      <button class="identify">
        <xsl:attribute name="onclick">
          <xsl:value-of 
            select="concat('GET(&quot;/link/',$url,'&quot;)')"/>
        </xsl:attribute>
        <xsl:attribute name="title">
          <xsl:value-of select="$strings/machine/identify_title"/>
        </xsl:attribute>
        <xsl:value-of select="$strings/machine/identify"/>
      </button>
    </xsl:if>
    
  </xsl:template>
  
  <!-- Processor information. -->
  <xsl:template name="processor">

    <table>
      <tr>
        <th><xsl:value-of select="$strings/processor/count"/></th>
        <td><xsl:value-of select="cpucount"/></td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/processor/cores"/></th>
        <td><xsl:value-of select="corecount"/></td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/processor/model"/></th>
        <td>
          <xsl:apply-templates select="speed" mode="units"/>
          <xsl:text> </xsl:text>
          <xsl:value-of select="cpu_type"/>
        </td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/processor/code"/></th>
        
        <td>
          <xsl:value-of select="cpucode"/>
          <xsl:if test="wikichip">
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of 
                  select="concat('GET(&quot;/link/https://en.wikichip.org/w/index.php?title=',wikichip, '&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/processor/wikichip_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/processor/wikichip"/>
            </button>
          </xsl:if>
        </td>
      </tr>
    </table>

    <p class="description">
      <xsl:copy-of select="$strings/processor/description/node()"/>

      <xsl:if test="wikichip">
        <xsl:copy-of select="$strings/processor/description_wikichip/node()"/>
      </xsl:if>
    </p>
    
  </xsl:template>
  
  <!-- RAM information. -->
  <xsl:template name="ram">

    <table>
      <tr>
        <th><xsl:value-of select="$strings/ram/total"/></th>
        <td>
          <xsl:apply-templates select="ram" mode="units"/>
      
          <div class="appleresources">
            <p>
              <xsl:choose> 
              
                <!-- If memory is upgradeable, say show and provide a link
                     to instructions from Apple. -->
                <xsl:when test="upgradeable = 'true'">  
                   
                  <strong>
                    <xsl:value-of select="$strings/ram/upgradeable"/>
                  </strong>
                  <button>
                    <xsl:attribute name="onclick">
                      <xsl:value-of 
                        select="concat('GET(&quot;/link/',memoryupgradeinstructionsurl,'&quot;)')"/>
                    </xsl:attribute>
                    <xsl:attribute name="title">
                      <xsl:value-of select="$strings/ram/upgradeable_instructions_title"/>
                    </xsl:attribute>
                    <xsl:value-of select="$strings/ram/upgradeable_instructions"/>
                  </button>

                </xsl:when>
                
                <!-- Better hope you bought enough RAM! -->
                <xsl:otherwise>
                  <strong>
                    <xsl:value-of select="$strings/ram/not_upgradeable"/>
                  </strong>
                </xsl:otherwise>
                
              </xsl:choose>

            </p>
          </div>
          
        </td>
      </tr>
      <tr>
        <th colspan="2">
          <xsl:value-of select="$strings/ram/memorybanks"/>
        </th>
      </tr>
      <tr>        
        <td colspan="2">
          <ul>
            <xsl:for-each select="memorybanks/memorybank">
            
              <!-- Add a list item for each DIMM. -->
              <li>

                <div class="DIMMWrapper">

                  <!-- Make it kinda look like a DIMM. -->
                  <img src="/image/DIMM.png">
                    <xsl:attribute name="alt">
                      <xsl:value-of select="$strings/ram/dimm_image"/>
                    </xsl:attribute>
                  </img>

                  <!-- Put the specs on top of the image. -->
                  <div class="dimm">
                    <p>
                      <xsl:value-of select="identifier"/>
                    </p>
                    <p>
                      <xsl:apply-templates select="size" mode="units"/>
                      <xsl:text> </xsl:text>
                      <xsl:value-of select="type"/>
                      <xsl:text> </xsl:text>
                      <xsl:apply-templates select="speed" mode="units"/>
                      <xsl:text> </xsl:text>
                      <xsl:value-of select="status"/>
                    </p>
                  </div>
                </div>
              </li>
            </xsl:for-each>
          </ul>
        </td>
      </tr>
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/ram/description/node()"/>
      
      <xsl:if test="upgradeable = 'true'">  
        <xsl:copy-of select="$strings/ram/description_upgradeable/node()"/>
      </xsl:if>
    </p>
    
  </xsl:template>
  
  <!-- Battery information. -->
  <xsl:template name="battery">

    <xsl:variable name="batteryhealth" select="$strings/batteryhealth"/>

    <xsl:variable name="healthvalue" select="batteryhealth"/>
    
    <table>
      <tr>
        <th><xsl:value-of select="$strings/battery/health"/></th>
        <td>
          <xsl:value-of select="$batteryhealth/value[@key = $healthvalue]"/>
          
          <!-- Provide a button to Apple resources about battery health. -->
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/link/https://support.apple.com/HT204054")</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/battery/more_information_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/battery/more_information"/>
          </button>
        </td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/battery/cycle_count"/></th>
        <td>
          <xsl:value-of select="batterycyclecount"/>

          <div class="meter">
            <xsl:call-template name="meter">
              <xsl:with-param name="percentage" select="batterypercent"/>
              <xsl:with-param 
                name="units" select="$strings/battery/percent_used"/>
            </xsl:call-template>
          </div>
          
          <!-- Provide a button to Apple resources about battery cycles. -->
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/link/https://support.apple.com/HT201585")</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/battery/about_cycles_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/battery/about_cycles"/>
          </button>
        </td>
      </tr>
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/battery/description/node()"/>
    </p>

  </xsl:template>
  
  <!-- Video information. -->
  <xsl:template match="video">
  
    <!-- Make a little table and show all GPUs and displays associated with
         each GPU. -->
    <table class="tablelist">
      <tr>
        <!-- Need to manually add a vertical border. -->
        <th class="verticalborder">
          <xsl:call-template name="plural">
            <xsl:with-param name="keys" select="$strings/video"/>
            <xsl:with-param name="key">
              <xsl:text>gpu</xsl:text>
            </xsl:with-param>
            <xsl:with-param name="value" select="exslt:node-set(count(gpu))"/>
          </xsl:call-template>
        </th>
        <th><xsl:value-of select="$strings/video/displays"/></th>
      </tr>
      
      <xsl:for-each select="gpu">
        <!-- Need to manually add borders. -->
        <tr class="horizontalborder">
          
          <td class="verticalborder">
            <strong><xsl:value-of select="name"/></strong>
            <br/>
            <xsl:choose>
              <xsl:when test="bus = 'spdisplays_builtin'">
                <xsl:value-of select="$strings/video/integrated"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="$strings/video/discrete"/>
              </xsl:otherwise>
            </xsl:choose>
            <br/>
            <xsl:if test="vram">
              <xsl:apply-templates select="vram" mode="units"/>
              <xsl:text> VRAM</xsl:text>
            </xsl:if>
            <xsl:if test="bus = 'spdisplays_builtin'">
              <br/>
              
              <!-- Show a button to get more information from Apple about
                   integrated video. -->
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET("/link/https://support.apple.com/HT204349")</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/video/about_integrated_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/video/about_integrated"/>
              </button>
            </xsl:if>
          </td>
          <td>
            <ul>
          
              <!-- Report displays, if any. -->
              <xsl:choose>
              
                <xsl:when test="count(displays/display) &gt; 0">
                  
                  <xsl:for-each select="displays/display">
                    <li>
                      <img>
                        <xsl:attribute name="src">
                          <xsl:value-of 
                            select="concat('/image/display/', name)"/>
                        </xsl:attribute>
                        <xsl:attribute name="alt">
                          <xsl:value-of select="$strings/video/display_icon"/>
                        </xsl:attribute>
                      </img>

                      <p>
                        <xsl:value-of select="name"/>
                      </p>
                      <p>
                        <xsl:value-of select="resolution"/>
                      </p>
                    </li>
                  </xsl:for-each>

                </xsl:when>
                
                <xsl:otherwise>
                  <li>
                    <p>
                      <xsl:value-of select="$strings/video/no_displays"/>
                    </p>
                  </li>
                </xsl:otherwise>
                
              </xsl:choose>
            </ul>
          </td>
        </tr>
      </xsl:for-each>
    </table>
      
    <p class="description">
      <xsl:copy-of select="$strings/video/description/node()"/>
      
      <xsl:if test="count(gpu) &gt; 1">
        <xsl:copy-of select="$strings/video/description_multiple/node()"/>
      </xsl:if>
      
      <xsl:if test="count(gpu[bus = 'spdisplays_builtin']) &gt; 0"> 
        <xsl:copy-of select="$strings/video/description_integrated/node()"/>
      </xsl:if>
    </p>

  </xsl:template>

  <!-- System software. -->
  <xsl:template match="systemsoftware">
  
    <table>
      <tr>
        <td rowspan="5">
          <img>
            <xsl:attribute name="src"> 
              <xsl:value-of 
                select="concat('http://localhost:', $port, '/image/os/', version)"/>
            </xsl:attribute>
            <xsl:attribute name="alt">
              <xsl:value-of select="$strings/system_software/icon"/>
            </xsl:attribute>
          </img>
        </td>
        <!-- Provide some space for the header. -->
        <td>
          <xsl:text>&#160;</xsl:text>
          <br/>
          <xsl:text>&#160;</xsl:text>
          <br/>
        </td>
        <td>
          <xsl:text>&#160;</xsl:text>
          <br/>
          <xsl:text>&#160;</xsl:text>
          <br/>
        </td>        
      </tr>
      <tr>
        <th>
          <xsl:value-of select="$strings/system_software/name"/>
        </th>
        <td><xsl:value-of select="name"/></td>
      </tr>
      <tr>
        <th>
          <xsl:value-of select="$strings/system_software/version"/>
        </th>
        <td><xsl:value-of select="version"/></td>
      </tr>
      <tr>      
        <th>
          <xsl:value-of select="$strings/system_software/build"/>
        </th>
        <td><xsl:value-of select="build"/></td>
      </tr>
      <tr>
        <th>
          <xsl:value-of select="$strings/system_software/uptime"/>
        </th>
        <td><xsl:apply-templates select="uptime"/></td>
      </tr>
    </table>

    <p class="description">
      <xsl:copy-of select="$strings/system_software/description/node()"/>
    </p>

  </xsl:template>

  <!-- Uptime. -->
  <xsl:template match="uptime">

    <xsl:variable name="days" select=". div 24"/>
    <xsl:variable name="hours" select=". mod 24"/>
    
    <xsl:choose>
    
      <xsl:when test="$days &gt;= 1">
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/uptime_human"/>
          <xsl:with-param name="key" select="'day'"/>
          <xsl:with-param name="value" select="$days"/>
        </xsl:call-template>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/uptime_human"/>
          <xsl:with-param name="key" select="'hour'"/>
          <xsl:with-param name="value" select="$hours"/>
        </xsl:call-template>
      </xsl:otherwise>
      
    </xsl:choose>
    
  </xsl:template>
  
  <!-- Configuration files. -->
  <xsl:template match="configurationfiles">
  
    <xsl:if test="count(*)">
    
      <table>
      
        <xsl:if test="etcsysctlconfexists">
          <tr>
            <th>
              <xsl:value-of 
                select="$strings/configuration/unexpected_file"/>
            </th>
            <td>/etc/sysctl.conf</td>
            <td>
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET("/reveal//etc/sysctl.conf")</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/configuration/reveal_in_finder_title"/>
                </xsl:attribute>
                <xsl:value-of 
                  select="$strings/configuration/reveal_in_finder"/>
              </button>
            </td>
          </tr>
        </xsl:if>
        
        <xsl:if test="etclaunchdconfexists">
          <tr>
            <th>
              <xsl:value-of 
                select="$strings/configuration/unexpected_file"/>
            </th>
            <td>/etc/launchd.conf</td>
            <td>
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET("/reveal//etc/launchd.conf")</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/configuration/reveal_in_finder_title"/>
                </xsl:attribute>
                <xsl:value-of 
                  select="$strings/configuration/reveal_in_finder"/>
              </button>
            </td>
          </tr>
        </xsl:if>

        <xsl:if test="hostscount or hostscorrupt">
          <tr>
            <th>
              <xsl:value-of select="$strings/configuration/hosts_count"/>
            </th>
            <td>
              <xsl:value-of select="hostscount"/>
              <xsl:if test="hostscorrupt">
                <xsl:text> - </xsl:text>
                <xsl:value-of 
                  select="$strings/configuration/hosts_corrupt"/>
              </xsl:if>
            </td>
            <td>
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET("/reveal//etc/hosts")</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/configuration/reveal_in_finder_title"/>
                </xsl:attribute>
                <xsl:value-of 
                  select="$strings/configuration/reveal_in_finder"/>
              </button>
            </td>
          </tr>
        </xsl:if>

        <!-- Use a for-each just to shorten the context. -->
        <xsl:for-each select="unexpectedsudoerssize">
        
          <xsl:variable 
            name="format"
            select="$strings/configuration/unexpected_sudoers_size_format"/>
            
          <tr>
            <th>
              <xsl:value-of 
                select="$strings/configuration/unexpected_sudoers_size"/>
            </th>
            <td>
              <xsl:call-template name="format">
                <xsl:with-param name="format" select="$format"/>
                <xsl:with-param name="value" select="size|expectedsize"/>
              </xsl:call-template>
            </td>
            <td>
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET("/reveal/etc/sudoers")</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/configuration/reveal_in_finder_title"/>
                </xsl:attribute>
                <xsl:value-of 
                  select="$strings/configuration/reveal_in_finder"/>
              </button>
            </td>
          </tr>
        </xsl:for-each>

      </table>
    
    </xsl:if>
        
    <p class="description">
      <xsl:copy-of select="$strings/configuration/description/node()"/>
    </p>

  </xsl:template>
  
  <!-- Go ahead and assume size and expected size should always have 
       units. -->
  <xsl:template match="size|expectedsize">
  
    <xsl:apply-templates select="." mode="units"/>
    
  </xsl:template>
  
  <!-- Print startup items. -->
  <xsl:template match="startupitems">
  
    <xsl:if test="count(startupitem) &gt; 0">

      <table class="tablelist">

        <xsl:for-each select="startupitem">

          <tr class="horizontalborder">
          
            <td class="header">
              <xsl:value-of select="$strings/startup_items/name"/>
              <br/>
              <xsl:value-of select="$strings/startup_items/path"/>
              <xsl:if test="version">
                <br/>
                <xsl:value-of select="$strings/startup_items/version"/>
              </xsl:if>
            </td>
            <td>
              <xsl:value-of select="name"/>
              <br/>
              <xsl:value-of select="path"/>
              <xsl:if test="version">
                <xsl:value-of select="$strings/startup_items/version"/>
                <br/>
                <xsl:value-of select="version"/>
              </xsl:if>
            </td>
          </tr>
        
        </xsl:for-each>
      
      </table>
      
    </xsl:if>
        
  </xsl:template>

  <!-- Print Fonts. -->
  <xsl:template match="fonts">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/fonts/name"/></th>
        <th><xsl:value-of select="$strings/fonts/path"/></th>
      </tr>
      <xsl:for-each select="font">
        <tr>
          <td><xsl:value-of select="name"/></td>
          <td><xsl:value-of select="path"/></td>
        </tr>
      </xsl:for-each>
    </table>
        
    <button>
      <xsl:attribute name="onclick">
        <xsl:text>GET("/openapplication//Applications/Font Book.app")</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="title">
        <xsl:value-of select="$strings/fonts/open_fontbook_title"/>
      </xsl:attribute>
      <xsl:value-of 
        select="$strings/fonts/open_fontbook"/>
    </button>

    <p class="description">
      <xsl:copy-of select="$strings/fonts/description/node()"/>
    </p>

  </xsl:template>
  
  <!-- Print a virtual memory usage information. -->
  <xsl:template match="vm">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/vm/available_ram"/></th>
        <td><xsl:apply-templates select="availableram" mode="units"/></td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/vm/free_ram"/></th>
        <td><xsl:apply-templates select="freeram" mode="units"/></td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/vm/used_ram"/></th>
        <td><xsl:apply-templates select="usedram" mode="units"/></td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/vm/cached_files"/></th>
        <td><xsl:apply-templates select="filecache" mode="units"/></td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/vm/swap_used"/></th>
        <td><xsl:apply-templates select="swapused" mode="units"/></td>
      </tr>
    </table>
      
    <p class="description">
      <xsl:copy-of select="$strings/vm/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print diagnostics information. -->
  <xsl:template match="diagnostics">
  
    <table class="tablelist">
    
      <xsl:apply-templates select="event"/>
      
    </table>

    <xsl:if test="insufficientpermissions = 'true'">
      <p>
        <strong>
          <xsl:value-of 
            select="$strings/diagnostics/insufficientpermissions"/>
        </strong>
      </p>
    </xsl:if>

    <p class="description">
      <xsl:copy-of select="$strings/diagnostics/description/node()"/>
      
      <xsl:if test="count(event[type &lt; 4]) &gt; 0">
        <xsl:copy-of 
          select="$strings/diagnostics/description_reports/node()"/>
      </xsl:if>
    </p>

  </xsl:template>

  <!-- Print a diagnostics event. -->
  <xsl:template match="event">
  
    <xsl:variable name="type" select="type"/>
    <xsl:variable 
      name="eventtype" 
      select="$strings/diagnostics/types/value[@key = $type]"/>
    
    <tr class="horizontalborder">
      
      <td class="header">
        <xsl:value-of select="$strings/diagnostics/type"/>
        <br/>
        <xsl:value-of select="$strings/diagnostics/date"/>
        
        <xsl:choose>
        
          <xsl:when test="type = 5"/>
          <xsl:when test="type = 8">
            <br/>
            <xsl:value-of select="$strings/diagnostics/details"/>
          </xsl:when>
          
          <xsl:when test="type = 10">
            <br/>
            <xsl:value-of select="$strings/diagnostics/reason"/>
          </xsl:when>

          <xsl:otherwise>
            <br/>
            <xsl:value-of select="$strings/diagnostics/application"/>
          </xsl:otherwise>
          
        </xsl:choose>
        
      </td>
      <td>
        
        <xsl:value-of select="$eventtype"/>
        
        <xsl:choose>
        
          <xsl:when test="type = 4"/>
          <xsl:when test="type = 5"/>
          <xsl:when test="type = 7"/>
          <xsl:when test="type = 8"/>
          <xsl:when test="type = 9"/>
          <xsl:when test="type = 10"/>
            
          <xsl:otherwise>
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of 
                  select="concat('GET(&quot;/open/',path,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/diagnostics/open_report_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/diagnostics/open_report"/>
            </button>
          </xsl:otherwise>
          
        </xsl:choose>
        
        <br/>
        <xsl:apply-templates select="date"/>
        <xsl:choose>
        
          <xsl:when test="type = 'Self test - failed'"/>
          
          <xsl:otherwise>
            <br/>
            <xsl:apply-templates select="name"/>
          </xsl:otherwise>
          
        </xsl:choose>
      
      </td>
    </tr>
    
    <!-- If I have information, add it. -->
    <xsl:if test="information">
      <tr>
      
        <!-- Make sure to use a description type to vertically line up
             the text properly. -->
        <td class="description header">
          <xsl:value-of select="$strings/diagnostics/details"/>
        </td>
        <td class="description">
          <xsl:apply-templates select="information"/>
        </td>
      </tr>
    </xsl:if>
        
    <xsl:if test="type = 10 and code = -60">
      <tr>
        <td class="disk_repair" colspan="2">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/link/https://support.apple.com/guide/disk-utility/repair-a-disk-dskutl1040/mac");</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/diagnostics/disk_repair_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/diagnostics/disk_repair"/>
          </button>
        </td>
      </tr>
    </xsl:if>
    
  </xsl:template>

  <!-- Physical drive information. -->
  <xsl:template match="disk">
  
    <xsl:apply-templates select="drive[mechanism != 'diskimage']"/>
    
    <p class="description">
      <xsl:copy-of select="$strings/drive/description/node()"/>
    </p>

  </xsl:template>

  <!-- Disk image information. -->
  <xsl:template name="diskimage">
  
    <xsl:if 
      test="count(/etrecheck/disk/drive[mechanism = 'diskimage']) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">diskimage</xsl:with-param>
        <xsl:with-param name="contents">
        
          <xsl:apply-templates 
            select="/etrecheck/disk/drive[mechanism = 'diskimage']"/>
          
          <p class="description">
            <xsl:copy-of select="$strings/diskimage/description/node()"/>
          </p>

        </xsl:with-param>
      </xsl:call-template>
      
    </xsl:if>
    
  </xsl:template>

  <!-- Drive information. -->
  <xsl:template match="drive">
  
    <xsl:variable name="type" select="type"/>
    <xsl:variable name="mechanism" select="mechanism"/>
    
    <table class="tablelist">
      <xsl:if test="mechanism != 'diskimage'">
        <tr>
          <th colspan="3">
            <xsl:value-of select="model"/>
          </th>
        </tr>
      </xsl:if>
      
      <tr class="horizontalborder">
   
        <td>
          <img>
            <xsl:attribute name="src">
              <xsl:value-of select="concat('/image/device/', device)"/>
            </xsl:attribute>
            <xsl:attribute name="alt">
              <xsl:value-of select="$strings/drive/icon"/>
            </xsl:attribute>
          </img>
        </td>
        <td class="header">
          <xsl:value-of select="$strings/drive/device"/>
            
          <br/>
          <xsl:value-of select="$strings/drive/size"/>
          <br/>
          <xsl:value-of select="$strings/drive/bus"/>
            
          <xsl:if test="busspeed">
            <br/>
            <xsl:value-of select="$strings/drive/busspeed"/>
          </xsl:if>

          <xsl:if test="type">
            <br/>
            <xsl:value-of select="$strings/drive/type"/>
          </xsl:if>
          
          <xsl:if test="mechanism != 'unknown'">
            <br/>
            <xsl:value-of select="$strings/drive/medium"/>
          </xsl:if>
          
          <xsl:if test="TRIM">
            <br/>
            <xsl:value-of select="$strings/drive/TRIM"/>
          </xsl:if>
          
          <xsl:choose>
            <xsl:when test="smartstatus = 'Verified'"/>
            <xsl:when test="smartstatus = 'Not Supported'"/>
            <xsl:otherwise>
              <br/>
              <xsl:value-of select="$strings/drive/smart_status"/>
            </xsl:otherwise>
          </xsl:choose>

        </td>
        <td>
          <xsl:value-of select="device"/>
            
          <br/>
          <xsl:apply-templates select="size" mode="units"/>
          <br/>
          <xsl:value-of select="bus"/>
            
          <xsl:if test="busspeed">
            <br/>
            <xsl:value-of select="busspeed"/>
          </xsl:if>

          <xsl:if test="type">
            <br/>
            <xsl:value-of select="$strings/drive/types/value[@key = $type]"/>
          </xsl:if>
          
          <xsl:if test="mechanism != 'unknown'">
            <br/>
            <xsl:value-of
              select="$strings/drive/mechanisms/value[@key = $mechanism]"/>
          </xsl:if>
          
          <xsl:if test="TRIM">
            <br/>
            <xsl:apply-templates select="TRIM"/>
          </xsl:if>
          
          <xsl:choose>
            <xsl:when test="smartstatus = 'Verified'"/>
            <xsl:when test="smartstatus = 'Not Supported'"/>
            <xsl:otherwise>
              <br/>
              <xsl:value-of select="smartstatus"/>
            </xsl:otherwise>
          </xsl:choose>

        </td>
      </tr>
      
      <xsl:if test="volumes/device">
        <tr class="horizontalborder">
          <td class="partitions" colspan="3">
            <p><xsl:value-of select="$strings/drive/partitions"/></p>
            <xsl:apply-templates select="volumes" mode="partition"/>
          </td>
        </tr>
      </xsl:if>
    </table>
     
  </xsl:template>

  <!-- A mini-list of devices. -->
  <xsl:template match="volumes" mode="partition">
  
    <ul>
      <xsl:apply-templates select="device" mode="partition"/>
    </ul>
    
  </xsl:template>
  
  <!-- A mini-list of devices. -->
  <xsl:template match="device" mode="partition">
  
    <xsl:variable name="device" select="."/>
    
    <xsl:variable 
      name="volume" 
      select="/etrecheck/virtualvolume/volume[device = $device]"/>
    
    <li>
      <a href="#">
        <xsl:attribute name="onclick">
          <xsl:value-of select="concat('showPartition(event, &quot;',$device, '&quot;)')"/>
        </xsl:attribute>
        
        <xsl:value-of select="$device"/>
      </a>
      
      <xsl:text> - </xsl:text>

      <xsl:choose>
        <xsl:when test="$volume/name">
          <xsl:value-of select="$volume/name"/>
          <xsl:text> </xsl:text>
        </xsl:when>
        
        <xsl:when test="$volume/type">
          <xsl:text>[</xsl:text>
          <xsl:value-of 
            select="$strings/volume/types/value[@key = $volume/type]"/>
          <xsl:text>] </xsl:text>
        </xsl:when>
        
        <xsl:otherwise/>
      </xsl:choose>
      
      <xsl:choose>
        <xsl:when test="$volume/used &gt; 0">
          <xsl:apply-templates select="$volume/used" mode="units"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates select="$volume/size" mode="units"/>
        </xsl:otherwise>
      </xsl:choose>
      
      <xsl:if test="$volume/filesystem">
        <xsl:text> </xsl:text>
        <xsl:value-of select="$volume/filesystem"/>
      </xsl:if>
      
      <xsl:if test="$volume/volumes/device">
        <br/>
        <p><xsl:value-of select="$strings/volume/partitions"/></p>
        <xsl:apply-templates select="$volume/volumes" mode="partition"/>
      </xsl:if>
    </li>
    
  </xsl:template>

  <!-- Volume information. -->
  <xsl:template match="virtualvolume" mode="top">
  
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">volumes</xsl:with-param>
      <xsl:with-param name="control">
      
        <button id="show_unmounted_volumes">
          <xsl:attribute name="onclick">
            <xsl:text>showUnmountedVolumes()</xsl:text>
          </xsl:attribute>
          <xsl:value-of select="$strings/volume/show_unmounted_volumes"/>
        </button>

        <button id="hide_unmounted_volumes">
          <xsl:attribute name="onclick">
            <xsl:text>hideUnmountedVolumes()</xsl:text>
          </xsl:attribute>
          <xsl:value-of select="$strings/volume/hide_unmounted_volumes"/>
        </button>
      </xsl:with-param>
      
      <xsl:with-param name="contents">
        <xsl:call-template name="volumes"/>
      </xsl:with-param>
    </xsl:call-template>

  </xsl:template>
  
  <!-- Volume information. -->
  <xsl:template name="volumes">
  
    <xsl:apply-templates select="volume"/>

    <p class="description">
      <xsl:copy-of select="$strings/volume/description/node()"/>
    </p>

  </xsl:template>

  <!-- Virtual volume information. -->
  <xsl:template match="volume">
  
    <xsl:variable name="type" select="type"/>

    <table class="tablelist">
      <xsl:attribute name="id">
        <xsl:value-of select="device"/>
      </xsl:attribute>
    
      <xsl:choose>
    
        <!-- Assign a notmounted class so that these volumes can be shown
             or hidden as requested. -->
        <xsl:when test="mountpoint">
          <xsl:attribute name="class">
            <xsl:text>tablelist</xsl:text>
          </xsl:attribute>
        </xsl:when>
        
        <xsl:otherwise>
          <xsl:attribute name="class">
            <xsl:text>tablelist notmounted</xsl:text>
          </xsl:attribute>
        </xsl:otherwise>
      </xsl:choose>
      
      <tr>
        <th colspan="3">
          
          <xsl:choose>
            <xsl:when test="name">
              <xsl:value-of select="name"/>
            </xsl:when>
            
            <xsl:otherwise>
              <xsl:value-of select="device"/>
            </xsl:otherwise>
            
          </xsl:choose>
          
        </th>
      </tr>
      <tr class="horizontalborder">
        <td>
          <img>
            <xsl:attribute name="src">
              <xsl:value-of select="concat('/image/device/', device)"/>
            </xsl:attribute>
            <xsl:attribute name="alt">
              <xsl:value-of select="$strings/volume/icon"/>
            </xsl:attribute>
          </img>
        </td>
        <td class="header">
          <xsl:value-of select="$strings/volume/device"/>
          
          <xsl:if test="filesystem">
            <br/>
            <xsl:value-of select="$strings/volume/filesystem"/>
          </xsl:if>
          
          <br/>
          <xsl:value-of select="$strings/volume/mount_point"/>
          
          <xsl:if test="type">
            <br/>
            <xsl:value-of select="$strings/volume/type"/>
          </xsl:if>
          
          <br/>
          <xsl:value-of select="$strings/volume/size"/>
          
          <xsl:if test="used &gt; 0">
            <br/>
            <xsl:value-of select="$strings/volume/used"/>
          </xsl:if>

          <xsl:if test="free and mountpoint">
            <br/>
            <xsl:value-of select="$strings/volume/free"/>
          </xsl:if>

          <xsl:if test="errors">   
            <br/>
            <xsl:value-of select="$strings/volume/errors"/>
          </xsl:if>

          <xsl:if test="encryption">
            <br/>
            <xsl:value-of select="$strings/volume/encryption_status"/>
          </xsl:if>

        </td>
        <td>
          <xsl:value-of select="device"/>
          
          <xsl:if test="filesystem">
            <br/>
            <xsl:value-of select="filesystem"/>
          </xsl:if>
          
          <br/>
          <xsl:choose>
            <xsl:when test="mountpoint">
              <xsl:value-of select="mountpoint"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="$strings/volume/not_mounted"/>
            </xsl:otherwise>
          </xsl:choose>
            
          <xsl:if test="type">
            <br/>
            <xsl:value-of 
              select="$strings/volume/types/value[@key = $type]"/>
          </xsl:if>
          
          <br/>
          <xsl:apply-templates select="size" mode="units"/>
          
          <xsl:if test="used &gt; 0">

            <br/>
            <xsl:apply-templates select="used" mode="units"/>

          </xsl:if>

          <xsl:if test="free and mountpoint">

            <br/>
            <xsl:apply-templates select="free" mode="units"/>
            <xsl:call-template name="freespace_gauge">
              <xsl:with-param name="size" select="size"/>
              <xsl:with-param name="free" select="free"/>
            </xsl:call-template>

          </xsl:if>

          <xsl:if test="errors">
   
            <br/>
            <xsl:call-template name="plural">
              <xsl:with-param name="keys" select="$strings/volume"/>
              <xsl:with-param name="key">
                <xsl:text>errors</xsl:text>
              </xsl:with-param>
              <xsl:with-param name="value" select="errors"/>
            </xsl:call-template>

          </xsl:if>

          <xsl:if test="encryption">
            <xsl:variable 
              name="encryptionstatus" select="encryption/status"/>
            
            <br/>
            <xsl:value-of 
              select="$strings/volume/encryption/value[@key = $encryptionstatus]"/>
              
            <xsl:if test="$encryptionstatus != 'encrypted'">
              <div class="progress">
                <xsl:call-template name="progress">
                  <xsl:with-param 
                    name="percentage" select="encryption/progress"/>
                  <xsl:with-param 
                    name="units" select="$strings/volume/percent_done"/>
                </xsl:call-template>
              </div>
            </xsl:if>
          </xsl:if>

        </td>
      </tr>
      
      <xsl:if test="volumes/device">
        <tr>
          <td class="partitions" colspan="3">
            <p><xsl:value-of select="$strings/volume/partitions"/></p>
            <xsl:apply-templates select="volumes" mode="partition"/>
          </td>
        </tr>
      </xsl:if>

    </table>
    
  </xsl:template>

  <!-- Print Time Machine information. -->
  <xsl:template match="timemachine">
  
    <table class="tablelist">
      <tr>
        <th colspan="3">
          <xsl:value-of select="$strings/tm/configuration"/>
        </th>
      </tr>
      
      <xsl:choose>
        <xsl:when test="notconfigured = 'true'">

          <tr class="horizontalborder">
            <td>
              <img src="/image/timemachine/icon">
                <xsl:attribute name="alt">
                  <xsl:value-of select="$strings/tm/tm_icon"/>
                </xsl:attribute>
              </img>
            </td>
            <td colspan="2">
              <xsl:value-of select="$strings/tm/notconfigured"/>
              
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET("/openapplication//System/Library/PreferencePanes/TimeMachine.prefPane")</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/tm/fix_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/tm/fix"/>
              </button>
            </td>
          </tr>

        </xsl:when>
        
        <xsl:otherwise>
        
          <tr class="horizontalborder">
            <td>
              <img src="/image/timemachine/icon">
                <xsl:attribute name="alt">
                  <xsl:value-of select="$strings/tm/tm_icon"/>
                </xsl:attribute>
              </img>
            </td>
            <td class="header">
              <xsl:if test="skipsystemfiles">
                <xsl:value-of select="$strings/tm/skip_system_files"/>
                <br/>
              </xsl:if>
              <xsl:if test="mobilebackups">
                <xsl:value-of select="$strings/tm/mobile_backups"/>
                <br/>
              </xsl:if>
              <xsl:value-of select="$strings/tm/auto_backup"/>
            </td>
            <td>
              <xsl:if test="skipsystemfiles">
                <xsl:apply-templates select="skipsystemfiles"/>
                <br/>
              </xsl:if>
              <xsl:if test="mobilebackups">
                <xsl:apply-templates select="mobilebackups"/>
                <br/>
              </xsl:if>
              <xsl:apply-templates select="autobackup"/>
              
              <xsl:if test="autobackup != 'true'">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:text>GET("/openapplication//System/Library/PreferencePanes/TimeMachine.prefPane")</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/tm/fix_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/tm/fix"/>
                </button>
              </xsl:if>
            </td>
          </tr>
          
          <tr>
            <th colspan="3">
              <xsl:value-of select="$strings/tm/drives_backed_up"/>
            </th>
          </tr>
          
          <xsl:apply-templates 
            select="volumesbeingbackedup/volume" mode="timemachine"/>
        
          <tr>
            <th colspan="3">
              <xsl:value-of select="$strings/tm/destinations"/>
            </th>
          </tr>
          
          <xsl:apply-templates select="destinations/destination"/>
        
        </xsl:otherwise>
      </xsl:choose>
      
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/tm/description/node()"/>
      
      <xsl:choose>
        <xsl:when test="notconfigured = 'true'">
          <xsl:copy-of select="$strings/tm/description_tmno/node()"/>
        </xsl:when>
        <xsl:when test="autobackup != 'true'">
          <xsl:copy-of select="$strings/tm/description_noauto/node()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy-of select="$strings/tm/description_tmyes/node()"/>
        </xsl:otherwise>
      </xsl:choose>
    </p>

  </xsl:template>

  <!-- Print a Time Machine volume. -->
  <xsl:template match="volume" mode="timemachine">
  
    <!-- This should always have a bottom border. -->
    <tr class="horizontalborder">
      <td>
        <img>
          <xsl:attribute name="src">
            <xsl:value-of select="concat('/image/timemachine/', name)"/>
          </xsl:attribute>
          <xsl:attribute name="alt">
            <xsl:value-of select="$strings/volume/icon"/>
          </xsl:attribute>
        </img>
      </td>
      <td class="header">
        <xsl:value-of select="$strings/tm/drive_name"/>
        <br/>
        <xsl:value-of select="$strings/tm/drive_size"/>
        <br/>
        <xsl:value-of select="$strings/tm/space_used"/>
      </td>
      <td>
        <xsl:value-of select="name"/>
        <br/>
        <xsl:apply-templates select="size" mode="units"/>
        <br/>
        <xsl:apply-templates select="used" mode="units"/>
      </td>
    </tr>

  </xsl:template>

  <!-- Print a Time Machine destination. -->
  <xsl:template match="destination">
  
    <!-- This should always have a bottom border. -->
    <tr class="horizontalborder">
      <td>
        <xsl:if test="position() = last()">
          <xsl:attribute name="class">
            <xsl:text>bottom_left</xsl:text>
          </xsl:attribute>
        </xsl:if>
        <img>
          <xsl:attribute name="src">
            <xsl:value-of select="concat('/image/timemachine/', name)"/>
          </xsl:attribute>
          <xsl:attribute name="alt">
            <xsl:value-of select="$strings/volume/icon"/>
          </xsl:attribute>
        </img>
      </td>
      <td class="header">
        <xsl:value-of select="$strings/tm/drive_name"/>
        <br/>
        <xsl:value-of select="$strings/tm/type"/>
        <br/>
        <xsl:value-of select="$strings/tm/last_used"/>
        <br/>
        <xsl:value-of select="$strings/tm/total_size"/>
        <br/>
        <xsl:value-of select="$strings/tm/number_of_backups"/>
        <br/>
        <xsl:value-of select="$strings/tm/oldest_backup"/>
        <br/>
        <xsl:value-of select="$strings/tm/last_backup"/>
      </td>
      <td>
        <xsl:value-of select="name"/>
        <br/>
        <xsl:value-of select="type"/>
        <br/>
        <xsl:apply-templates select="lastused"/>
        <br/>
        <xsl:apply-templates select="size" mode="units"/>
        <br/>
        <xsl:value-of select="count"/>
        <br/>
        <xsl:value-of select="oldestbackup"/>
        <br/>
        <xsl:value-of select="lastbackup"/>
      </td>
    </tr>

  </xsl:template>

  <!-- Print a network information. -->
  <xsl:template match="network">
  
    <table class="tablelist">
      <tr>
        <th colspan="4">
          <xsl:value-of select="$strings/network/interfaces"/>
        </th>
      </tr>

      <xsl:for-each select="interfaces/networkinterface">

        <xsl:variable name="interface" select="interface"/>
        <xsl:variable 
          name="modes" 
          select="/etrecheck/hardware/wireless[name = $interface]"/>
        
        <tr class="horizontalborder">
          <td class="header">
            <xsl:value-of select="$strings/network/interface"/>
          </td>
          <td colspan="3">
            <xsl:value-of select="interface"/>
          </td>
        </tr>

        <tr>
          <td class="header">
            <xsl:value-of select="$strings/network/name"/>
          </td>
          <td colspan="3">
            <xsl:value-of select="name"/>
          </td>
        </tr>
        
        <xsl:if test="$modes">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/network/modes"/>
            </td>
            <td colspan="3">
              <xsl:value-of select="$modes/modes"/>
            </td>
          </tr>
        </xsl:if>
        
        <xsl:if test="count(ipv4addresses/address) &gt; 0">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/network/ipv4addresses"/>
            </td>
            <td colspan="3">
              <xsl:for-each select="ipv4addresses/address">
                <xsl:if test="position() &gt; 1">
                  <br/>
                </xsl:if>
                
                <xsl:value-of select="."/>
              </xsl:for-each>
            </td>
          </tr>
        </xsl:if>
        
        <xsl:if test="count(ipv6addresses/address) &gt; 0">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/network/ipv6addresses"/>
            </td>
            <td colspan="3">
              <xsl:for-each select="ipv6addresses/address">
                <xsl:if test="position() &gt; 1">
                  <br/>
                </xsl:if>
                
                <xsl:value-of select="."/>
              </xsl:for-each>
            </td>
          </tr>
        </xsl:if>
        
        <xsl:if test="proxyautodiscovery = 'true'">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/network/proxyautodiscovery"/>
            </td>
            <td colspan="3">
              <xsl:apply-templates select="proxyautodiscovery"/>
            </td>
          </tr>
        </xsl:if>
        
        <xsl:if test="proxyautoconfig = 'true'">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/network/proxyautoconfig"/>
            </td>
            <td colspan="3">
              <xsl:value-of select="proxyautoconfigurl"/>
            </td>
          </tr>
        </xsl:if>

        <xsl:if test="count(proxies/proxy) &gt; 0">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/network/proxies"/>
            </td>
            <td colspan="3">
              <xsl:for-each select="proxies/proxy">
                <xsl:if test="position() &gt; 1">
                  <br/>
                </xsl:if>
                
                <xsl:value-of select="type"/>
                
                <xsl:if test="address">
                  <br/>
                  
                  <xsl:if test="user">
                    <xsl:value-of select="address"/>
                    <xsl:text>@</xsl:text>
                  </xsl:if>
                  
                  <xsl:value-of select="address"/>
                  
                  <xsl:if test="port">
                    <xsl:text>:</xsl:text>
                    <xsl:value-of select="port"/>
                  </xsl:if>
                </xsl:if>
              </xsl:for-each>
            </td>
          </tr>
        </xsl:if>

      </xsl:for-each>

      <xsl:if test="icloudfree or icloudpendingfiles">
        <tr class="horizontalborder">
          <th colspan="4">
            <xsl:value-of select="$strings/network/icloud"/>
          </th>
        </tr>
      </xsl:if>
      
      <xsl:if test="icloudfree">
        <tr class="horizontalborder">
          <td class="header">
            <xsl:value-of select="$strings/network/icloudfree"/>
          </td>
          <td colspan="3">
            <xsl:apply-templates 
              select="icloudfree" mode="units"/>
            <xsl:value-of select="$strings/available"/>
          </td>
        </tr>
      </xsl:if>
      
      <xsl:if test="icloudpendingfiles">
        
        <tr class="horizontalborder">
          <td class="header">
            <xsl:value-of select="$strings/network/icloudstatus"/>
          </td>
          <td colspan="3">
            <xsl:call-template name="plural">
              <xsl:with-param name="keys" select="$strings/network"/>
              <xsl:with-param name="key">
                <xsl:text>icloudpendingfiles</xsl:text>
              </xsl:with-param>
              <xsl:with-param 
                name="value" 
                select="exslt:node-set(count(icloudpendingfiles//file))"/>
            </xsl:call-template>
          </td>
        </tr>
        
        <xsl:apply-templates 
          select="icloudpendingfiles/directory"/>

      </xsl:if>
      
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/network/description/node()"/>

      <xsl:if test="icloudfree">
        <xsl:copy-of select="$strings/network/description_icloud/node()"/>
      </xsl:if>

      <xsl:if test="icloudpendingfiles">
        <xsl:copy-of select="$strings/network/description_icloudpendingfiles/node()"/>
      </xsl:if>
      
      <xsl:choose>
        <xsl:when test="count(interfaces/networkinterface[proxyautodiscovery = 'true']) &gt; 0">
          <xsl:copy-of select="$strings/network/description_proxy/node()"/>
        </xsl:when>
        <xsl:when test="count(interfaces/networkinterface[proxyautoconfig = 'true']) &gt; 0">
          <xsl:copy-of select="$strings/network/description_proxy/node()"/>
        </xsl:when>
        <xsl:when test="count(interfaces//proxy) &gt; 0">
          <xsl:copy-of select="$strings/network/description_proxy/node()"/>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>
    </p>

  </xsl:template>

  <!-- Emit an iCloud pending file directory. -->
  <xsl:template match="icloudpendingfiles/directory">

    <tr>
      <th colspan="2">
        <em><xsl:value-of select="$strings/network/iclouddrive"/></em>
        <xsl:value-of select="name"/>        
      </th>
      <th><xsl:value-of select="$strings/network/status"/></th>
      <th><xsl:value-of select="$strings/network/progress"/></th>
    </tr>

    <xsl:apply-templates select="file"/>

  </xsl:template>

  <!-- Emit an iCloud pending file. -->
  <xsl:template match="icloudpendingfiles/directory/file">

    <tr class="horizontalborder">
      
      <td colspan="2"><xsl:value-of select="name"/></td>
              
      <xsl:choose>
      
        <xsl:when test="action">
          <xsl:variable name="action" select="action"/>
          
          <td>
            <xsl:value-of 
              select="$strings/network/action/value[@key = $action]"/>
          </td>
          <td>
            <div class="progress">
              <xsl:call-template name="progress">
                <xsl:with-param name="percentage" select="percentage"/>
                <xsl:with-param 
                  name="units" select="$strings/network/percent"/>
              </xsl:call-template>
            </div>
          </td>
        </xsl:when>
       
        <xsl:otherwise>
          <td colspan="2"/>
        </xsl:otherwise>
       
      </xsl:choose>
        
    </tr>

  </xsl:template>
  
  <!-- Top processes by network use. -->
  <xsl:template match="networkusage">
  
    <xsl:variable name="thirdparty">
      <xsl:for-each select="process">
        <xsl:variable name="path" select="path"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when 
              test="/etrecheck/header/context/process[path = $path]/source">
              <xsl:value-of 
                select="/etrecheck/header/context/process[path = $path]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
          
        <xsl:if test="$path and $source != 'Apple'">
          <xsl:value-of select="$source"/>
        </xsl:if>
      </xsl:for-each>
    </xsl:variable>
    
    <table>
      <tr>
        <th colspan="2">
          <xsl:value-of select="$strings/networkprocesses/process"/>
        </th>
        <th><xsl:value-of select="$strings/networkprocesses/source"/></th>
        <th><xsl:value-of select="$strings/networkprocesses/input"/></th>
        <th colspan="2"><xsl:value-of select="$strings/networkprocesses/output"/></th>
      </tr>
      <xsl:for-each select="process">
        <xsl:variable name="path" select="path"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when 
              test="/etrecheck/header/context/process[path = $path]/source">
              <xsl:value-of 
                select="/etrecheck/header/context/process[path = $path]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:variable 
          name="identifier" 
          select="/etrecheck/header/context/process[path = $path]/identifier"/>

        <tr>
          <td>
            <img class="appicon">
              <xsl:attribute name="src"> 
                <xsl:value-of 
                  select="concat('http://localhost:', $port, '/image/process/', $identifier)"/>
              </xsl:attribute>
              <xsl:attribute name="alt">
                <xsl:value-of select="$strings/networkprocesses/app_icon"/>
              </xsl:attribute>
            </img>
          </td>
          <td><xsl:value-of select="name"/></td>
          <td><xsl:value-of select="$source"/></td>
          <td><xsl:apply-templates select="inputsize" mode="units"/></td>
          <td><xsl:apply-templates select="outputsize" mode="units"/></td>
          <td>
            <xsl:if test="$path and $source != 'Apple'">
              <button>
                <xsl:attribute name="onclick">
                  <xsl:value-of 
                    select="concat('GET(&quot;/reveal/',$path,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/cpu/reveal_in_finder_title"/>
                </xsl:attribute>
                <xsl:value-of 
                  select="$strings/cpu/reveal_in_finder"/>
              </button>
            </xsl:if>
          </td>
        </tr>
      </xsl:for-each>
    </table>
          
    <p class="description">
      <xsl:copy-of select="$strings/networkprocesses/description/node()"/>

      <xsl:if test="string-length($thirdparty)">
        <xsl:copy-of select="$strings/networkprocesses/description_thirdparty/node()"/>
      </xsl:if>
    </p>

  </xsl:template>

  <!-- Security information. -->
  <xsl:template match="security">
  
    <xsl:if test="outdated = 'true'">
      <button>
        <xsl:attribute name="onclick">
          <xsl:text>GET("/fix/criticalupdates")</xsl:text>
        </xsl:attribute>
        <xsl:attribute name="title">
          <xsl:value-of select="$strings/security/install_updates_title"/>
        </xsl:attribute>
        <xsl:value-of 
          select="$strings/security/install_updates"/>
      </button>
    </xsl:if>

    <table>
      <tr>
        <th><xsl:value-of select="$strings/security/system"/></th>
        <th/>
        <th><xsl:value-of select="$strings/security/status"/></th>
      </tr>
      <tr>
        <td>
          <xsl:value-of select="$strings/security/gatekeeper"/>
        </td>
        <td>  
          <!-- Provide a button to Apple information about gatekeeper. -->
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/link/https://support.apple.com/HT202491")</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of 
                select="$strings/security/about_gatekeeper_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/security/about_gatekeeper"/>
          </button>
        </td>
        <td id="gatekeeperstatus">
          <xsl:variable name="gatekeeperstatus" select="gatekeeper/status"/>
          
          <xsl:value-of 
            select="$strings/security/gatekeeper_setting/value[@key = $gatekeeperstatus]"/>
          <xsl:if test="gatekeeper/status = 'Anywhere'">
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET("/fix/gatekeeper")</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of 
                  select="$strings/security/fix_gatekeeper_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/security/fix_gatekeeper"/>
            </button>
          </xsl:if>
        </td>
      </tr>
      <xsl:if test="/etrecheck/systemsoftware/majorversion &gt; 10">
        <tr>
          <td>
            <xsl:value-of select="$strings/security/sip"/>  
          </td>
          <td>
            <!-- Provide a button to Apple information about SIP. -->
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET("/link/https://support.apple.com/HT204899")</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/security/about_sip_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/security/about_sip"/>
            </button>
          </td>
          <td>
            <xsl:choose>
              <xsl:when test="SIP = 'enabled'">
                <xsl:value-of select="$strings/security/sip_enabled"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="SIP"/>
              </xsl:otherwise>
            </xsl:choose>

            <xsl:if test="sip != 'enabled'">
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET("/fix/SIP")</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/security/fix_sip_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/security/fix_sip"/>
              </button>
            </xsl:if>
          </td>
        </tr>
      </xsl:if>
    </table>

    <table>
      <tr>
        <th/>
        <th/>
        <th class="merged" colspan="2">
          <xsl:value-of select="$strings/security/versions"/>
        </th>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/security/system"/></th>
        <th/>
        <th><xsl:value-of select="$strings/security/currentversion"/></th>
        <th><xsl:value-of select="$strings/security/installedversion"/></th>
      </tr>
      <tr>
        <td>
          <xsl:value-of select="$strings/security/gatekeeper"/>
        </td>
        <td>  
          <!-- Provide a button to Apple information about gatekeeper. -->
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/link/https://support.apple.com/HT202491")</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/security/about_gatekeeper_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/security/about_gatekeeper"/>
          </button>
        </td>
        <td><xsl:apply-templates select="gatekeeper/currentversion"/></td>
        <td><xsl:apply-templates select="gatekeeper/installedversion"/></td>
      </tr>
      <tr>
        <td>
          <xsl:value-of select="$strings/security/xprotect"/>
        </td>
        <td>
          <!-- Provide a button to Apple information about XProtect. -->
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/link/https://support.apple.com/HT201940")</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/security/about_xprotect_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/security/about_xprotect"/>
          </button>
        </td>
        <td><xsl:apply-templates select="xprotect/currentversion"/></td>
        <td><xsl:apply-templates select="xprotect/installedversion"/></td>
      </tr>
      
      <xsl:if test="/etrecheck/systemsoftware/majorversion &gt; 10">
        <tr>
          <td><xsl:value-of select="$strings/security/mrt"/></td>
          <td/>
          <td><xsl:apply-templates select="mrt/currentversion"/></td>
          <td><xsl:apply-templates select="mrt/installedversion"/></td>
        </tr>
      </xsl:if>
      
    </table>

    <p class="description">
      <xsl:copy-of select="$strings/security/description/node()"/>
      
      <xsl:copy-of 
        select="$strings/security/description_gatekeeper/node()"/>

      <xsl:if test="gatekeeper/status = 'Anywhere'">
        <xsl:copy-of 
          select="$strings/security/description_gatekeeper_fix/node()"/>
      </xsl:if>

      <xsl:copy-of select="$strings/security/description_sip/node()"/>
      <xsl:if test="sip != 'enabled'">
        <xsl:copy-of 
          select="$strings/security/description_sip_fix/node()"/>
      </xsl:if>

      <xsl:copy-of select="$strings/security/description_versions/node()"/>
      <xsl:if test="outdated = 'true'">
        <xsl:copy-of 
          select="$strings/security/description_versions_fix/node()"/>
      </xsl:if>
    </p>
    
  </xsl:template>

  <!-- Adware information. -->
  <xsl:template match="adware" mode="top">

    <xsl:if test="count(launchdfiles/launchdfile) &gt; 0 or count(safariextensions/extension) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <xsl:template match="adware">

    <table class="tablelist">
      <xsl:for-each select="launchdfiles/launchdfile">

        <xsl:variable name="adware" select="adware"/>

        <tbody>
          <xsl:attribute name="class">
            <xsl:value-of select="identifier"/>
            <xsl:text> group</xsl:text>
          </xsl:attribute>
          
          <tr class="horizontalborder">
            <td class="header">
              <xsl:value-of select="$strings/adware/type"/>
            </td>
            <td><xsl:value-of select="$strings/adware/launchd"/></td>
            <td>
              <button class="remove">
                <xsl:attribute name="onclick">
                  <xsl:value-of 
                    select="concat('GET(&quot;/launchd/removeadware/',identifier,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/adware/remove_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/adware/remove"/>
              </button>
              <button class="report">
                <xsl:attribute name="onclick">
                  <xsl:text>GET("/reportproblem")</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/adware/reportproblem_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/adware/reportproblem"/>
              </button>
            </td>
          </tr>
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/adware/file"/>
            </td>
            <td colspan="2"><xsl:value-of select="path"/></td>
          </tr>
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/adware/executable"/>
            </td>
            <td colspan="2">
              <xsl:value-of select="executable"/>
              <xsl:for-each select="arguments/argument">
                <xsl:value-of select="concat(' ', .)"/>
              </xsl:for-each>
            </td>
          </tr>
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/adware/reason"/>
            </td>
            <td colspan="2">
              <xsl:value-of 
                select="$strings/adware/reason_text/value[@key = $adware]"/>
            </td>
          </tr>
        </tbody>
      </xsl:for-each>

      <xsl:for-each select="safariextensions/extension">
        <tbody>
          <xsl:attribute name="class">
            <xsl:value-of select="identifier"/>
            <xsl:text> group</xsl:text>
          </xsl:attribute>
          
          <tr class="horizontalborder">
            <td class="header">
              <xsl:value-of select="$strings/adware/type"/>
            </td>
            <td><xsl:value-of select="$strings/adware/safariextension"/></td>
            <td>
              <button class="remove">
                <xsl:attribute name="onclick">
                  <xsl:value-of 
                    select="concat('GET(&quot;/safari/removeadware/',identifier,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/adware/remove_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/adware/remove"/>
              </button>
            </td>
          </tr>
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/adware/file"/>
            </td>
            <td colspan="2"><xsl:value-of select="displayname"/></td>
          </tr>
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/adware/reason"/>
            </td>
            <td colspan="2">
              <xsl:value-of 
                select="$strings/adware/reason_text/value[@key = $adware]"/>
            </td>
          </tr>
        </tbody>
      </xsl:for-each>

    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/adware/description/node()"/>
    </p>
    
  </xsl:template>

  <!-- Unsigned files information. -->
  <xsl:template match="unsigned" mode="top">

    <xsl:if test="count(launchdfiles/launchdfile[apple != 'true']) &gt; 0 or count(safariextensions/extension) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <!-- Print unsigned files. -->
  <xsl:template match="unsigned">
  
    <table class="tablelist">

      <xsl:apply-templates select="launchdfiles" mode="unsigned"/>
      <xsl:apply-templates select="safariextensions" mode="unsigned"/>
      
    </table>
        
    <p class="description">
      <xsl:copy-of select="$strings/unsigned/description/node()"/>

      <xsl:if test="count(launchdfiles//details['invalid' = substring(., string-length(.) - 7)]) = 0">
        <xsl:copy-of select="$strings/unsigned/description_legitimate/node()"/>
      </xsl:if>
      
      <xsl:copy-of select="$strings/unsigned/description_research/node()"/>
      <xsl:copy-of select="$strings/unsigned/description_action/node()"/>
      
      <xsl:if test="count(safariextensions) &gt; 0">
        <xsl:copy-of select="$strings/unsigned/description_safari/node()"/>
      </xsl:if>
    </p>
           
  </xsl:template>

  <!-- Print launchd files. -->
  <xsl:template match="launchdfiles" mode="unsigned">
  
    <tr>
      <th colspan="2">
        <xsl:value-of select="$strings/unsigned/launchdfiles"/>
      </th>
    </tr>

    <xsl:apply-templates select="launchdfile" mode="unsigned"/>

  </xsl:template>
  
  <!-- Print an unsigned launchd file. -->
  <xsl:template match="launchdfile" mode="unsigned">
  
    <xsl:variable name="status" select="status"/>
    
    <xsl:variable name="details" select="details"/>

    <xsl:variable name="ascsearch">
      <xsl:value-of select="$strings/launchd/ascsearch"/>
      <xsl:text>type=discussion&amp;showAnsweredFirst=true&amp;q=</xsl:text>
      <xsl:value-of select="label"/>
      <xsl:text>&amp;sort=updatedDesc&amp;currentPage=1&amp;includeResultCount=true</xsl:text>
    </xsl:variable>
    
    <xsl:if test="apple != 'true'">
      <xsl:variable name="remove">
        <xsl:choose>
          <xsl:when test="details = 'whitelist'">
            <xsl:text>remove_confirm</xsl:text>
          </xsl:when>
          <xsl:when test="details = 'whitelistprefix'">
            <xsl:text>remove_confirm</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>remove_unsigned</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      
      <tbody>
        <xsl:attribute name="class">
          <xsl:value-of select="identifier"/>
          <xsl:text> group</xsl:text>
        </xsl:attribute>
        
        <tr class="horizontalborder">
          <td class="header">
            <xsl:value-of select="$strings/unsigned/launchd"/>
          </td>
          <td class="data">
            <xsl:value-of select="path"/>
          </td>
        </tr>
            
        <xsl:if test="executable">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/unsigned/executable"/>
            </td>
            <td class="data">
              <xsl:value-of select="executable"/>
              <xsl:for-each select="arguments/argument">
                <xsl:value-of select="concat(' ', .)"/>
              </xsl:for-each>
            </td>
          </tr>
        </xsl:if>
            
        <tr>
          <td class="header">
            <xsl:value-of select="$strings/unsigned/status"/>
          </td>
          <td class="data status">
            <xsl:value-of 
              select="$strings/launchd/status_text/value[@key = $status]"/>
          </td>
        </tr>
        <tr>
          <td class="header">
            <xsl:value-of select="$strings/unsigned/installdate"/>
          </td>
          <td class="data">
            <xsl:value-of select="installdate"/>
          </td>
        </tr>
        <xsl:if test="details">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/unsigned/details"/>
            </td>
            <td class="data">
              <xsl:value-of 
                select="$strings/unsigned/details_text/value[@key = $details]"/>
            </td>
          </tr>
        </xsl:if>
        <tr>
          <td class="control" colspan="2">
            <button class="reveal_config">
              <xsl:attribute name="onclick">
                <xsl:value-of 
                  select="concat('GET(&quot;/reveal/',path,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/launchd/reveal_config_title"/>
              </xsl:attribute>
              <xsl:value-of 
                select="$strings/launchd/reveal_config"/>
            </button>
            <button class="reveal_executable">
              <xsl:attribute name="onclick">
                <xsl:value-of 
                  select="concat('GET(&quot;/reveal/',executable,'&quot;)')"/>
              </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/launchd/reveal_executable_title"/>
            </xsl:attribute>
              <xsl:value-of 
                select="$strings/launchd/reveal_executable"/>
            </button>
            <button class="lookup">
              <xsl:attribute name="onclick">
                <xsl:value-of 
                  select="concat('GET(&quot;/link/',$ascsearch,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/launchd/lookup_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/launchd/lookup"/>
            </button>
            <button class="report">
              <xsl:attribute name="onclick">
                <xsl:text>GET("/reportproblem")</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/unsigned/reportproblem_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/unsigned/reportproblem"/>
            </button>
            <button class="remove">
              <xsl:attribute name="onclick">
                <xsl:value-of 
                  select="concat('GET(&quot;/launchd/',$remove,'/',identifier,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/unsigned/remove_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/unsigned/remove"/>
            </button>
          </td>
        </tr>
      </tbody>
    </xsl:if>
    
  </xsl:template>

  <!-- Print Safari extensions. -->
  <xsl:template match="safariextensions" mode="unsigned">
  
    <tr>
      <th colspan="2">
        <div class="headerwrapper">
          <xsl:value-of select="$strings/unsigned/safariextensions"/>
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/openapplication//Applications/Safari.app")</xsl:text>
            </xsl:attribute>
            <xsl:value-of 
              select="$strings/safari/open_safari"/>
          </button>
        </div>
      </th>
    </tr>

    <xsl:apply-templates select="extension" mode="unsigned"/>
    
  </xsl:template>
  
  <!-- Print an unsigned Safari extension. -->
  <xsl:template match="extension" mode="unsigned">
  
    <tr class="horizontalborder">
      <td class="header">
        <xsl:value-of select="$strings/unsigned/safariextension"/>
      </td>
      <td class="data">
        <xsl:value-of select="displayname"/>
      </td>
    </tr>
    <tr>
      <td class="header">
        <xsl:value-of select="$strings/unsigned/installdate"/>
      </td>
      <td class="data">
        <xsl:value-of select="installdate"/>
      </td>
    </tr>

  </xsl:template>

  <!-- Kernel extensions. -->
  <xsl:template match="kernelextensions">
  
    <table class="tablelist">

      <xsl:for-each select="directory">
        <xsl:variable name="name" select="name"/>
        <xsl:variable 
          name="iconname" 
          select="/etrecheck/header/context/application[name = $name]/name"/>

        <tr>
          <th>
            <xsl:value-of select="$strings/kernelextensions/container"/>
          </th>
          <th>
            <xsl:choose>
              
              <xsl:when test="name">
                <xsl:if test="$iconname">
                  <img class="appicon">
                    <xsl:attribute name="src"> 
                      <xsl:value-of 
                        select="concat('http://localhost:', $port, '/image/kernelapp/', $iconname)"/>
                    </xsl:attribute>
                    <xsl:attribute name="alt">
                      <xsl:value-of select="$strings/kernelextensions/app_icon"/>
                    </xsl:attribute>
                  </img>
                </xsl:if>
                
                <xsl:value-of select="name"/>
              </xsl:when>
              
              <xsl:otherwise>
                <xsl:value-of select="path"/>
              </xsl:otherwise>
              
            </xsl:choose>
            
            <xsl:choose>
            
              <xsl:when test="path = '/Library/Extensions'">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:text>GET("/openfolder_restricted//Library/Extensions")</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/kernelextensions/open_in_finder_title"/>
                  </xsl:attribute>
                  <xsl:value-of 
                    select="$strings/kernelextensions/open_in_finder"/>
                </button>
            
              </xsl:when>
            
              <xsl:otherwise>
              
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:value-of 
                      select="concat('GET(&quot;/reveal/',path,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/kernelextensions/reveal_in_finder_title"/>
                  </xsl:attribute>
                  <xsl:value-of 
                    select="$strings/kernelextensions/reveal_in_finder"/>
                </button>
                
              </xsl:otherwise>
            
            </xsl:choose>
            
          </th>
        </tr>

        <xsl:for-each select="extensions/extension">
        
          <xsl:variable name="status" select="status"/>
          
          <tr class="horizontalborder">
            <td class="header">
              <xsl:value-of select="$strings/kernelextensions/extension"/>
              <br/>
              <xsl:value-of select="$strings/kernelextensions/label"/>
              <br/>
              <xsl:value-of select="$strings/kernelextensions/version"/>
              <br/>
              <xsl:value-of select="$strings/kernelextensions/status"/>
            </td>
            <td>
              <xsl:value-of select="filename"/>
              <br/>
              <xsl:value-of select="bundleid"/>
              <br/>
              
              <xsl:if test="version">
                <xsl:value-of select="version"/>
              </xsl:if>
              <xsl:if test="osversion">
                <xsl:if test="version">
                  <xsl:text> - </xsl:text>
                </xsl:if>
                <xsl:value-of select="osversion"/>
              </xsl:if>
              
              <br/>
              <xsl:value-of 
                select="$strings/kernelextensions/status_text/value[@key = $status]"/>
            </td>
          </tr>
        </xsl:for-each>            
      </xsl:for-each>
        
    </table>
        
    <p class="description">
      <xsl:copy-of select="$strings/kernelextensions/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print system launch agents. -->
  <xsl:template match="systemlaunchagents">
  
    <xsl:call-template name="applelaunchdfiles"/>
    
    <p class="description">
      <xsl:copy-of select="$strings/systemlaunchagents/description/node()"/>
    </p>

  </xsl:template>
  
  <!-- Print system launch daemons. -->
  <xsl:template match="systemlaunchdaemons">
  
    <xsl:call-template name="applelaunchdfiles"/>
        
    <p class="description">
      <xsl:copy-of select="$strings/systemlaunchdaemons/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print Apple launchd files. -->
  <xsl:template name="applelaunchdfiles">
  
    <table class="tablelist">
  
      <xsl:apply-templates select="applelaunchdfile"/>
      
    </table>
    
  </xsl:template>

  <!-- Print Apple launchd files. -->
  <xsl:template match="applelaunchdfile">
  
    <xsl:variable name="type">
    
      <xsl:choose>
        <xsl:when test="name(..) = 'systemlaunchagents'">
          <xsl:text>agent</xsl:text>
        </xsl:when>
        <xsl:when test="name(..) = 'systemlaunchdaemons'">
          <xsl:text>daemon</xsl:text>
        </xsl:when>
      </xsl:choose>
      
    </xsl:variable>
    
    <xsl:variable name="status" select="status"/>
    
    <xsl:variable name="last_row">
      <xsl:choose>
        <xsl:when test="position() = last()">
          <xsl:text>true</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>false</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <tr>
      <xsl:if test="position() = last()">
        <xsl:attribute name="class">
          <xsl:text>last_child</xsl:text>
        </xsl:attribute>
      </xsl:if>
      
      <th>
        <xsl:value-of select="$strings/launchd/status"/>
      </th>
      <th>
        <xsl:value-of 
          select="$strings/launchd/status_text/value[@key = $status]"/>
        <xsl:text> - </xsl:text>
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/launchd"/>
          <xsl:with-param name="key" select="'apple_files'"/>
          <xsl:with-param name="value" select="count"/>
        </xsl:call-template>
      </th>
      <th>
        <div class="buttonwrapper">
          <button class="show_apple_files">
            <xsl:attribute name="onclick">
              <xsl:value-of 
                select="concat('showAppleFiles(this, &quot;', status, '&quot;,&quot;', $type, '&quot;,', $last_row, ')')"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/launchd/show"/>
          </button>
          <button class="hide_apple_files">
            <xsl:attribute name="onclick">
              <xsl:value-of 
                select="concat('hideAppleFiles(this, &quot;', status, '&quot;,&quot;', $type, '&quot;,', $last_row, ')')"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/launchd/hide"/>
          </button>
        </div>
      </th>
    </tr>

    <tr>
      <xsl:attribute name="class">
        <xsl:value-of select="concat('apple ', $status, ' ', $type)"/>
      </xsl:attribute>

      <th><xsl:value-of select="$strings/applelaunchd/author"/></th>
      <th><xsl:value-of select="$strings/applelaunchd/label"/></th>
      <th>
        <xsl:value-of select="$strings/launchd/install_date"/>
      </th>
    </tr>
      
    <xsl:for-each select="../launchdfile[status = $status]">

      <tr>
        <xsl:attribute name="class">
          <xsl:value-of select="concat('apple horizontalborder ', $status, ' ', $type)"/>
        </xsl:attribute>
        
        <td class="author"><xsl:value-of select="author"/></td>
        <td class="label">
          <xsl:value-of select="label"/>
        
          <xsl:if test="$status = 'failed'">
            <xsl:value-of 
              select="concat(' (', $strings/launchd/result, ' ', lastexitcode, ')')"/>
          </xsl:if>
          
        </td>
        
        <td class="installdate"><xsl:value-of select="installdate"/></td>
      </tr>

    </xsl:for-each>
    
  </xsl:template>

  <!-- Print launch agents. -->
  <xsl:template match="launchagents">
  
    <xsl:call-template name="launchdfiles"/>
        
    <p class="description">
      <xsl:copy-of select="$strings/launchagents/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print launch daemons. -->
  <xsl:template match="launchdaemons">
  
    <xsl:call-template name="launchdfiles"/>
        
    <p class="description">
      <xsl:copy-of select="$strings/launchdaemons/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print user launch agents. -->
  <xsl:template match="userlaunchagents">
  
    <xsl:call-template name="launchdfiles"/>
        
    <p class="description">
      <xsl:copy-of select="$strings/userlaunchagents/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print launchd filess. -->
  <xsl:template name="launchdfiles">
  
    <table class="tablelist">
  
      <xsl:apply-templates select="launchdfile"/>
      
    </table>
    
  </xsl:template>
  
  <!-- Print a launchd file. -->
  <xsl:template match="launchdfile">
  
    <xsl:variable name="status" select="status"/>
    
    <xsl:variable name="ascsearch">
      <xsl:value-of select="$strings/launchd/ascsearch"/>
      <xsl:text>type=discussion&amp;showAnsweredFirst=true&amp;q=</xsl:text>
      <xsl:value-of select="label"/>
      <xsl:text>&amp;sort=updatedDesc&amp;currentPage=1&amp;includeResultCount=true</xsl:text>
    </xsl:variable>

    <tbody>
      <xsl:attribute name="class">
        <xsl:value-of select="identifier"/>
        <xsl:text> group</xsl:text>
      </xsl:attribute>

      <tr class="horizontalborder">
        <td class="header label">
          <xsl:value-of select="$strings/launchd/filename"/>
        </td>
        <td class="verticalborder data">
          <xsl:value-of select="filename"/>
        </td>
      </tr>
      <tr>
        <td class="header label">
          <xsl:value-of select="$strings/launchd/label"/>
        </td>
        <td class="verticalborder data">
          <xsl:value-of select="label"/>
        </td>
      </tr>
      <tr>
        <td class="header label">
          <xsl:value-of select="$strings/launchd/status"/>
        </td>
        <td class="verticalborder status">
          <xsl:value-of 
            select="$strings/launchd/status_text/value[@key = $status]"/>
        </td>
      </tr>
      <tr>
        <td class="header label">
          <xsl:value-of select="$strings/launchd/author"/>
        </td>
        <td class="verticalborder">
          <xsl:value-of select="author"/>
        </td>
      </tr>
      <tr>
        <td class="header label">
          <xsl:value-of select="$strings/launchd/install_date"/>
        </td>
        <td class="verticalborder">
          <xsl:value-of select="installdate"/>
        </td>
      </tr>
      <tr class="buttonrow">
        <td class="verticalborder" colspan="2">
          <button>
            <xsl:attribute name="onclick">
              <xsl:value-of 
                select="concat('GET(&quot;/reveal/',path,'&quot;)')"/>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/launchd/reveal_config_title"/>
            </xsl:attribute>
            <xsl:value-of 
              select="$strings/launchd/reveal_config"/>
          </button>
          <button>
            <xsl:attribute name="onclick">
              <xsl:value-of 
                select="concat('GET(&quot;/reveal/',executable,'&quot;)')"/>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/launchd/reveal_executable_title"/>
            </xsl:attribute>
            <xsl:value-of 
              select="$strings/launchd/reveal_executable"/>
          </button>
          <button class="lookup">
            <xsl:attribute name="onclick">
              <xsl:value-of 
                select="concat('GET(&quot;/link/',$ascsearch,'&quot;)')"/>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/launchd/lookup_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/launchd/lookup"/>
          </button>
        </td>
      </tr>
    </tbody>
        
  </xsl:template>

  <!-- Print login items. -->
  <xsl:template match="loginitems" mode="top">
  
    <xsl:if test="count(loginitem) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
        
  </xsl:template>

  <xsl:template match="loginitems">
  
    <table class="tablelist">
      
      <xsl:apply-templates select="loginitem"/>
    
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/loginitems/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print a single login item. -->
  <xsl:template match="loginitem">
  
    <tr class="horizontalborder">

      <td class="header">
        <xsl:value-of select="$strings/loginitems/name"/>
      </td>
      <td>
        <xsl:value-of select="name"/>
      </td>
      <td>
        <xsl:choose>
          <xsl:when test="type = 'SMLoginItem'">
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of 
                  select="concat('GET(&quot;/reveal/',path,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/loginitems/reveal_in_finder_title"/>
              </xsl:attribute>
              <xsl:value-of 
                select="$strings/loginitems/reveal_in_finder"/>
            </button>
          </xsl:when>
          <xsl:otherwise>
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET("/openapplication//System/Library/PreferencePanes/Accounts.prefPane")</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of 
                  select="$strings/loginitems/open_users_and_groups_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/loginitems/open_users_and_groups"/>
            </button>
          </xsl:otherwise>
        </xsl:choose>        
      </td>
    </tr>
    
    <tr>
      <td class="header">
        <xsl:value-of select="$strings/loginitems/type"/>        
      </td>
      <td colspan="2">
        <xsl:value-of select="type"/>
      </td>
    </tr>
    
    <xsl:if test="signature">
      <tr>
        <td class="header">
          <xsl:value-of select="$strings/loginitems/author"/>
        </td>
        <td colspan="2">
          <xsl:value-of select="signature"/>
        </td>
      </tr>
    </xsl:if>

    <xsl:if test="installdate">
      <tr>
        <td class="header">
          <xsl:value-of select="$strings/loginitems/install_date"/>
        </td>
        <td colspan="2">
          <xsl:value-of select="installdate"/>
        </td>
      </tr>
    </xsl:if>
    <tr>
      <td class="header">
        <xsl:value-of select="$strings/loginitems/executable"/>
      </td>
      <td class="path" colspan="2">
        <xsl:value-of select="path"/>
      </td>
    </tr>
        
  </xsl:template>

  <!-- Print internet plugins. -->
  <xsl:template match="internetplugins" mode="top">
  
    <xsl:if test="count(plugin) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/openfolder//Library/Internet Plug-Ins")</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/plugins/open_in_finder_title"/>
            </xsl:attribute>
            <xsl:value-of 
              select="$strings/plugins/open_in_finder"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
        
  </xsl:template>

  <xsl:template match="internetplugins">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/plugins/name"/></th>
        <th><xsl:value-of select="$strings/plugins/version"/></th>
        <th><xsl:value-of select="$strings/plugins/date"/></th>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
        
    <p class="description">
      <xsl:copy-of select="$strings/internetplugins/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print user internet plugins. -->
  <xsl:template match="userinternetplugins" mode="top">
  
    <xsl:if test="count(plugin) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/openfolder/~/Library/Internet Plug-Ins")</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/plugins/open_in_finder_title"/>
            </xsl:attribute>
            <xsl:value-of 
              select="$strings/plugins/open_in_finder"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
        
  </xsl:template>

  <xsl:template match="userinternetplugins">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/plugins/name"/></th>
        <th><xsl:value-of select="$strings/plugins/version"/></th>
        <th><xsl:value-of select="$strings/plugins/date"/></th>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
        
    <p class="description">
      <xsl:copy-of select="$strings/userinternetplugins/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print audio plugins. -->
  <xsl:template match="audioplugins" mode="top">
  
    <xsl:if test="count(plugin) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/openfolder//Library/Audio/Plug-Ins")</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/plugins/open_in_finder_title"/>
            </xsl:attribute>
            <xsl:value-of 
              select="$strings/plugins/open_in_finder"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
        
  </xsl:template>

  <xsl:template match="audioplugins">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/plugins/name"/></th>
        <th><xsl:value-of select="$strings/plugins/version"/></th>
        <th><xsl:value-of select="$strings/plugins/date"/></th>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
        
    <p class="description">
      <xsl:copy-of select="$strings/audioplugins/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print user audio plugins. -->
  <xsl:template match="useraudioplugins" mode="top">
  
    <xsl:if test="count(plugin) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/openfolder/~/Library/Audio/Plug-Ins")</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/plugins/open_in_finder_title"/>
            </xsl:attribute>
            <xsl:value-of 
              select="$strings/plugins/open_in_finder"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
        
  </xsl:template>

  <xsl:template match="useraudioplugins">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/plugins/name"/></th>
        <th><xsl:value-of select="$strings/plugins/version"/></th>
        <th><xsl:value-of select="$strings/plugins/date"/></th>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
        
    <p class="description">
      <xsl:copy-of select="$strings/useraudioplugins/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print iTunes plugins. -->
  <xsl:template match="itunesplugins" mode="top">
  
    <xsl:if test="count(plugin) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/openfolder//Library/iTunes/iTunes Plug-ins")</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/plugins/open_in_finder_title"/>
            </xsl:attribute>
            <xsl:value-of 
              select="$strings/plugins/open_in_finder"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
        
  </xsl:template>

  <xsl:template match="itunesplugins">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/plugins/name"/></th>
        <th><xsl:value-of select="$strings/plugins/version"/></th>
        <th><xsl:value-of select="$strings/plugins/date"/></th>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
        
    <p class="description">
      <xsl:copy-of select="$strings/itunesplugins/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print user iTunes plugins. -->
  <xsl:template match="useritunesplugins" mode="top">
  
    <xsl:if test="count(plugin) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/openfolder/~/Library/iTunes/iTunes Plug-ins")</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/plugins/open_in_finder_title"/>
            </xsl:attribute>
            <xsl:value-of 
              select="$strings/plugins/open_in_finder"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
        
  </xsl:template>

  <xsl:template match="useritunesplugins">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/plugins/name"/></th>
        <th><xsl:value-of select="$strings/plugins/version"/></th>
        <th><xsl:value-of select="$strings/plugins/date"/></th>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
        
    <p class="description">
      <xsl:copy-of select="$strings/useritunesplugins/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print a single plugin. -->
  <xsl:template match="plugin">
  
    <tr>
      <td><xsl:value-of select="name"/></td>
      <td><xsl:value-of select="version"/></td>
      <td><xsl:value-of select="installdate"/></td>
    </tr>

  </xsl:template>

  <!-- Print Safari extensions. -->
  <xsl:template match="safariextensions" mode="top">
  
    <xsl:if test="count(extension) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/openapplication//Applications/Safari.app")</xsl:text>
            </xsl:attribute>
            <xsl:value-of 
              select="$strings/safari/open_safari"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
        
  </xsl:template>
  
  <xsl:template match="safariextensions">
  
    <table class="tablelist">
    
      <tr>
        <th colspan="2">
          <xsl:value-of select="$strings/safari/extension"/>
        </th>
        <th><xsl:value-of select="$strings/safari/install_date"/></th>
      </tr>
      
      <xsl:apply-templates select="extension"/>
    
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/safari/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print a single Safari extension. -->
  <xsl:template match="extension">
  
    <xsl:variable name="status" select="status"/>
    
    <tr>
      <xsl:attribute name="class">
        <xsl:value-of select="concat('horizontalborder ', identifier)"/>
      </xsl:attribute>

      <td class="header">
        <xsl:value-of select="$strings/safari/name"/>
        
        <xsl:if test="developer">
          <br/>
          <xsl:value-of select="$strings/safari/developer"/>
        </xsl:if>
        
        <xsl:if test="url">
          <br/>
          <xsl:value-of select="$strings/safari/url"/>
        </xsl:if>
        
       </td>
      <td>
        <xsl:value-of select="displayname"/>

        <xsl:if test="developer">
          <br/>
          <xsl:value-of select="developer"/>
        </xsl:if>
        
        <xsl:if test="url">
          <br/>
          <a href="#">
            <xsl:attribute name="onclick">
              <xsl:value-of 
                select="concat('GET(&quot;/link/',url,'&quot;)')"/>
            </xsl:attribute>
            <xsl:value-of select="url"/>
          </a>
        </xsl:if>

      </td>
      <td>
        <xsl:value-of select="installdate"/>
      </td>
    </tr>
        
  </xsl:template>

  <!-- Print Preference panes. -->
  <xsl:template match="preferencepanes" mode="top">
  
    <xsl:if test="count(preferencepane) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/openapplication//Applications/System Preferences.app")</xsl:text>
            </xsl:attribute>
            <xsl:value-of 
              select="$strings/preferencepane/open_system_preferences"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
        
  </xsl:template>

  <xsl:template match="preferencepanes">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/preferencepane/name"/></th>
        <th><xsl:value-of select="$strings/preferencepane/date"/></th>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
        
    <p class="description">
      <xsl:copy-of select="$strings/preferencepane/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print a single Preference pane. -->
  <xsl:template match="preferencepane">
  
    <tr>
      <td><xsl:value-of select="name"/></td>
      <td><xsl:value-of select="installdate"/></td>
    </tr>
        
  </xsl:template>

  <!-- Print software install information. -->
  <xsl:template match="install" mode="top">
  
    <xsl:if test="count(package) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
        
  </xsl:template>

  <xsl:template match="install">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/install/name"/></th>
        <th><xsl:value-of select="$strings/install/version"/></th>
        <th><xsl:value-of select="$strings/install/date"/></th>
      </tr>
    
      <xsl:apply-templates select="package"/>
    
    </table>
        
    <p class="description">
      <xsl:copy-of select="$strings/install/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print an install package. -->
  <xsl:template match="package">
  
    <tr>
      <td><xsl:value-of select="name"/></td>
      <td><xsl:value-of select="version"/></td>
      <td><xsl:value-of select="installdate"/></td>
    </tr>

  </xsl:template>

  <!-- Clean up information. -->
  <xsl:template match="cleanup">
  
    <xsl:variable 
      name="launchdfiles" 
      select="count(launchdfiles/file) &gt; 0"/>
    
    <xsl:variable 
      name="notificationspam" 
      select="count(notificationspam/spammer) &gt; 0"/>

    <xsl:variable 
      name="safariextensions" 
      select="count(safariextensions/extension) &gt; 0"/>

    <xsl:if test="launchdfiles or notificationspam or safariextensions">

      <table class="tablelist">
      
        <xsl:if test="launchdfiles">

          <xsl:for-each select="launchdfiles/launchdfile">
        
            <tbody>
              <xsl:attribute name="class">
                <xsl:value-of select="identifier"/>
                <xsl:text> group</xsl:text>
              </xsl:attribute>
              
              <tr class="horizontalborder">
                <td class="header">
                  <xsl:value-of select="$strings/cleanup/launchd_file"/>
                </td>
                <td colspan="2" class="data">
                  <xsl:value-of select="path"/>
                </td>
              </tr>
              <tr>
                <td class="header">
                  <xsl:value-of select="$strings/cleanup/missing_executable"/>
                </td>
                <td colspan="2" class="data">
                  <xsl:value-of select="executable"/>
                </td>
              </tr>
              <tr>
                <td colspan="3" class="control">
                  <button>
                    <xsl:attribute name="onclick">
                      <xsl:value-of 
                        select="concat('GET(&quot;/reveal/',path,'&quot;)')"/>
                    </xsl:attribute>
                    <xsl:attribute name="title">
                      <xsl:value-of select="$strings/cleanup/reveal_in_finder_title"/>
                    </xsl:attribute>
                    <xsl:value-of 
                      select="$strings/cleanup/reveal_in_finder"/>
                  </button>
                  <button>
                    <xsl:attribute name="onclick">
                      <xsl:value-of 
                        select="concat('GET(&quot;/launchd/cleanup/',identifier,'&quot;)')"/>
                    </xsl:attribute>
                    <xsl:attribute name="title">
                      <xsl:value-of select="$strings/cleanup/remove_title"/>
                    </xsl:attribute>
                    <xsl:value-of select="$strings/cleanup/remove"/>
                  </button>
                </td>
              </tr>
            </tbody>
            
          </xsl:for-each>
            
        </xsl:if>

        <xsl:if test="safariextensions">

          <xsl:for-each select="safariextensions/extension">
          
            <tbody>
              <xsl:attribute name="class">
                <xsl:value-of select="identifier"/>
                <xsl:text> group</xsl:text>
              </xsl:attribute>
              
              <tr class="horizontalborder">
                <td class="header">
                  <xsl:value-of 
                    select="$strings/cleanup/safari_extension"/>
                </td>
                <td colspan="2" class="data"><xsl:value-of select="displayname"/></td>
              </tr>
              <tr>
                <td colspan="3" class="control">
                  <button>
                    <xsl:attribute name="onclick">
                      <xsl:value-of 
                        select="concat('GET(&quot;/reveal/',path,'&quot;)')"/>
                    </xsl:attribute>
                    <xsl:attribute name="title">
                      <xsl:value-of select="$strings/cleanup/reveal_in_finder_title"/>
                    </xsl:attribute>
                    <xsl:value-of 
                      select="$strings/cleanup/reveal_in_finder"/>
                  </button>
                  <button>
                    <xsl:attribute name="onclick">
                      <xsl:value-of 
                        select="concat('GET(&quot;/safari/cleanup/',identifier,'&quot;)')"/>
                    </xsl:attribute>
                    <xsl:attribute name="title">
                      <xsl:value-of select="$strings/cleanup/remove_title"/>
                    </xsl:attribute>
                    <xsl:value-of select="$strings/cleanup/remove"/>
                  </button>
                </td>
              </tr>
            </tbody>

          </xsl:for-each>
            
        </xsl:if>
        
        <xsl:if test="notificationspam">
        
          <xsl:for-each select="notificationspam/spammer">

            <tr class="horizontalborder">
              <td class="header">
                <xsl:value-of select="$strings/cleanup/notification_spam"/>
              </td>
              <td class="data">
                <xsl:value-of select="name"/>
                <xsl:text> - </xsl:text>
                <xsl:call-template name="plural">
                  <xsl:with-param name="keys" select="$strings/cleanup"/>
                  <xsl:with-param 
                    name="key" select="'spamnotificationcount'"/>
                  <xsl:with-param name="value" select="count"/>
                </xsl:call-template>
              </td>
              <td class="control">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:value-of 
                      select="concat('GET(&quot;/cleanup/notificationspam/',name,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/cleanup/remove_spam_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/cleanup/remove"/>
                </button>
              </td>
            </tr>

          </xsl:for-each>
            
        </xsl:if>
        
      </table>
      
    </xsl:if>
        
    <p class="description">
      <xsl:copy-of select="$strings/cleanup/description/node()"/>
      
      <xsl:if test="launchdfiles">
        <xsl:copy-of select="$strings/cleanup/description_launchd/node()"/>
      </xsl:if>

      <xsl:if test="safariextensions">
        <xsl:copy-of select="$strings/cleanup/description_safari/node()"/>
      </xsl:if>

      <xsl:if test="notificationspam">
        <xsl:copy-of select="$strings/cleanup/description_spam/node()"/>
      </xsl:if>

      <xsl:copy-of select="$strings/cleanup/description_remove/node()"/>
    </p>

  </xsl:template>

  <!-- Show overall performance. -->
  <xsl:template name="performance">
  
    <xsl:variable 
      name="performance" select="/etrecheck/header/performance"/>

    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">performance</xsl:with-param>
      <xsl:with-param name="contents">
        <table class="tablelist">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/performance/systemload"/>
            </td>
            <td>
              <img id="loadgraph" src="/image/loadgraph.png">
                <xsl:attribute name="alt">
                  <xsl:value-of 
                    select="$strings/performance/loadgraph_alt"/>
                </xsl:attribute>
              </img>
            </td>
          </tr>
          <tr class="horizontalborder">
            <td class="header">
              <xsl:value-of select="$strings/performance/etrecheckruntime"/>
            </td>
            <td>
              <xsl:value-of select="/etrecheck/header/runtime"/>
              <xsl:text> </xsl:text>
              <xsl:value-of select="$strings/performance/runtimeunits"/>
            </td>
          </tr>
          <tr class="horizontalborder">
            <td class="header">
              <xsl:value-of 
                select="$strings/performance/etrecheckperformance"/>
            </td>
            <td>
              <xsl:value-of 
                select="$strings/performance_rating/value[@key = $performance]"/>
            </td>
          </tr>
        </table>
        
        <p class="description">
          <xsl:copy-of select="$strings/performance/description/node()"/>
        </p>
    
      </xsl:with-param>
    </xsl:call-template>
    
  </xsl:template>

  <!-- Print a CPU usage information. -->
  <xsl:template match="cpu">
  
    <xsl:variable name="thirdparty">
      <xsl:for-each select="process">
        <xsl:variable name="path" select="path"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when 
              test="/etrecheck/header/context/process[path = $path]/source">
              <xsl:value-of 
                select="/etrecheck/header/context/process[path = $path]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
          
        <xsl:if test="$path and $source != 'Apple'">
          <xsl:value-of select="$source"/>
        </xsl:if>
      </xsl:for-each>
    </xsl:variable>
    
    <table>
      <tr>
        <th colspan="2"><xsl:value-of select="$strings/cpu/process"/></th>
        <th><xsl:value-of select="$strings/cpu/source"/></th>
        <th colspan="2"><xsl:value-of select="$strings/cpu/usage"/></th>
      </tr>
      <xsl:for-each select="process">
        <xsl:variable name="path" select="path"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when 
              test="/etrecheck/header/context/process[path = $path]/source">
              <xsl:value-of 
                select="/etrecheck/header/context/process[path = $path]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:variable 
          name="identifier" 
          select="/etrecheck/header/context/process[path = $path]/identifier"/>

        <tr>
          <td>
            <img class="appicon">
              <xsl:attribute name="src"> 
                <xsl:value-of 
                  select="concat('http://localhost:', $port, '/image/process/', $identifier)"/>
              </xsl:attribute>
              <xsl:attribute name="alt">
                <xsl:value-of select="$strings/cpu/app_icon"/>
              </xsl:attribute>
            </img>
          </td>
          <td>
            <xsl:value-of select="name"/>
            
            <xsl:if test="count &gt; 1">
              <xsl:value-of select="concat(' (',count,')')"/>
            </xsl:if>
            
          </td>
          <td><xsl:value-of select="$source"/></td>
          <td><xsl:apply-templates select="usage" mode="units"/></td>
          <td>
            <xsl:if test="$path and $source != 'Apple'">
              <button>
                <xsl:attribute name="onclick">
                  <xsl:value-of 
                    select="concat('GET(&quot;/reveal/',$path,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/cpu/reveal_in_finder_title"/>
                </xsl:attribute>
                <xsl:value-of 
                  select="$strings/cpu/reveal_in_finder"/>
              </button>
            </xsl:if>
          </td>
        </tr>
      </xsl:for-each>
    </table>
      
    <p class="description">
      <xsl:copy-of select="$strings/cpu/description/node()"/>

      <xsl:if test="string-length($thirdparty)">
        <xsl:copy-of select="$strings/cpu/description_thirdparty/node()"/>
      </xsl:if>
    </p>

  </xsl:template>

  <!-- Print a memory usage information. -->
  <xsl:template match="memory">
  
    <xsl:variable name="thirdparty">
      <xsl:for-each select="process">
        <xsl:variable name="path" select="path"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when 
              test="/etrecheck/header/context/process[path = $path]/source">
              <xsl:value-of 
                select="/etrecheck/header/context/process[path = $path]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
          
        <xsl:if test="$path and $source != 'Apple'">
          <xsl:value-of select="$source"/>
        </xsl:if>
      </xsl:for-each>
    </xsl:variable>
    
    <table>
      <tr>
        <th colspan="2">
          <xsl:value-of select="$strings/memory/process"/>
        </th>
        <th><xsl:value-of select="$strings/memory/source"/></th>
        <th colspan="2"><xsl:value-of select="$strings/memory/usage"/></th>
      </tr>
      <xsl:for-each select="process">
        <xsl:variable name="path" select="path"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when 
              test="/etrecheck/header/context/process[path = $path]/source">
              <xsl:value-of 
                select="/etrecheck/header/context/process[path = $path]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:variable 
          name="identifier" 
          select="/etrecheck/header/context/process[path = $path]/identifier"/>
        
        <tr>
          <td>
            <img class="appicon">
              <xsl:attribute name="src"> 
                <xsl:value-of 
                  select="concat('http://localhost:', $port, '/image/process/', $identifier)"/>
              </xsl:attribute>
              <xsl:attribute name="alt">
                <xsl:value-of select="$strings/memory/app_icon"/>
              </xsl:attribute>
            </img>
          </td>
          <td>
            <xsl:value-of select="name"/>
            
            <xsl:if test="count &gt; 1">
              <xsl:value-of select="concat(' (',count,')')"/>
            </xsl:if>
            
          </td>
          <td><xsl:value-of select="$source"/></td>
          <td><xsl:apply-templates select="size" mode="units"/></td>
          <td>
            <xsl:if test="$path and $source != 'Apple'">
              <button>
                <xsl:attribute name="onclick">
                  <xsl:value-of 
                    select="concat('GET(&quot;/reveal/',$path,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/cpu/reveal_in_finder_title"/>
                </xsl:attribute>
                <xsl:value-of 
                  select="$strings/cpu/reveal_in_finder"/>
              </button>
            </xsl:if>
          </td>
        </tr>
      </xsl:for-each>
    </table>
      
    <p class="description">
      <xsl:copy-of select="$strings/memory/description/node()"/>

      <xsl:if test="string-length($thirdparty)">
        <xsl:copy-of select="$strings/cpu/description_thirdparty/node()"/>
      </xsl:if>
    </p>

  </xsl:template>

  <!-- Print a energy usage information. -->
  <xsl:template match="energy">
  
    <xsl:variable name="thirdparty">
      <xsl:for-each select="process">
        <xsl:variable name="path" select="path"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when 
              test="/etrecheck/header/context/process[path = $path]/source">
              <xsl:value-of 
                select="/etrecheck/header/context/process[path = $path]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
          
        <xsl:if test="$path and $source != 'Apple'">
          <xsl:value-of select="$source"/>
        </xsl:if>
      </xsl:for-each>
    </xsl:variable>
    
    <table>
      <tr>
        <th colspan="2">
          <xsl:value-of select="$strings/energy/process"/>
        </th>
        <th><xsl:value-of select="$strings/energy/source"/></th>
        <th colspan="2"><xsl:value-of select="$strings/energy/usage"/></th>
      </tr>
      <xsl:for-each select="process">
        <xsl:variable name="path" select="path"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when 
              test="/etrecheck/header/context/process[path = $path]/source">
              <xsl:value-of 
                select="/etrecheck/header/context/process[path = $path]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:variable 
          name="identifier" 
          select="/etrecheck/header/context/process[path = $path]/identifier"/>
        
        <tr>
          <td>
            <img class="appicon">
              <xsl:attribute name="src"> 
                <xsl:value-of 
                  select="concat('http://localhost:', $port, '/image/process/', $identifier)"/>
              </xsl:attribute>
              <xsl:attribute name="alt">
                <xsl:value-of select="$strings/energy/app_icon"/>
              </xsl:attribute>
            </img>
          </td>
          <td>
            <xsl:value-of select="name"/>
            
            <xsl:if test="count &gt; 1">
              <xsl:value-of select="concat(' (',count,')')"/>
            </xsl:if>
            
          </td>
          <td><xsl:value-of select="$source"/></td>
          <td><xsl:value-of select="energy"/></td>
          <td>
            <xsl:if test="$path and $source != 'Apple'">
              <button>
                <xsl:attribute name="onclick">
                  <xsl:value-of 
                    select="concat('GET(&quot;/reveal/',$path,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/cpu/reveal_in_finder_title"/>
                </xsl:attribute>
                <xsl:value-of 
                  select="$strings/cpu/reveal_in_finder"/>
              </button>
            </xsl:if>
          </td>
        </tr>
      </xsl:for-each>
    </table>
      
    <p class="description">
      <xsl:copy-of select="$strings/energy/description/node()"/>

      <xsl:if test="string-length($thirdparty)">
        <xsl:copy-of select="$strings/cpu/description_thirdparty/node()"/>
      </xsl:if>
    </p>

  </xsl:template>
  
  <!-- Print a EtreCheck deleted files. -->
  <xsl:template match="etrecheckdeletedfiles">
  
    <xsl:if test="count(deletedfile) &gt; 0">
      <xsl:call-template name="header"/>

      <table>
        <xsl:call-template name="tablestyle"/>
        
        <xsl:for-each select="deletedfile">
          <tr>
            <td>
              <xsl:call-template name="style"/>          
              <xsl:value-of select="date"/>
            </td>
            <td>
              <xsl:call-template name="style"/>          
              <xsl:value-of select="path"/>
            </td>
            <td>
              <xsl:call-template name="style"/>          
              <xsl:value-of select="reason"/>
            </td>
          </tr>
        </xsl:for-each>
        
      </table>
      
    </xsl:if>
      
  </xsl:template>

  <!-- Display a free space gauge. -->
  <xsl:template name="freespace_gauge">
    <xsl:param name="size"/>
    <xsl:param name="free"/>
    
    <xsl:variable name="expanded_size">
      <xsl:call-template name="expand">
        <xsl:with-param name="number" select="size"/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:variable name="expanded_free">
      <xsl:call-template name="expand">
        <xsl:with-param name="number" select="free"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable 
      name="percentage" 
      select="($expanded_size - $expanded_free) * 100 div $expanded_size"/>

    <div class="meter">
      <xsl:call-template name="meter">
        <xsl:with-param name="percentage" select="$percentage"/>
        <xsl:with-param name="units" select="$strings/volume/percent_used"/>
      </xsl:call-template>
    </div>
    
  </xsl:template>
    
  <!-- Display a meter -->
  <xsl:template name="meter">
    <xsl:param name="percentage"/>
    <xsl:param name="units"/>
    
    <div class="range">
    
      <xsl:if test="$percentage &gt; 5">
        <div class="level">
          <xsl:attribute name="style">
            <xsl:value-of 
              select="concat('width: ', $percentage, 'px;')"/>
            <xsl:choose>
              <xsl:when test="$percentage &gt; 85">
                <xsl:text>background: red</xsl:text>   
              </xsl:when>
              <xsl:when test="$percentage &gt; 70">
                <xsl:text>background: yellow</xsl:text>   
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>background: rgb(0, 242, 0)</xsl:text>   
              </xsl:otherwise>
            </xsl:choose>
          </xsl:attribute>
        </div>
      </xsl:if>
    
    </div>
    
    <span>
      <xsl:value-of select="concat(round($percentage), $units)"/>
    </span>
    
  </xsl:template>
  
  <!-- Display a progress bar -->
  <xsl:template name="progress">
    <xsl:param name="percentage"/>
    <xsl:param name="units"/>
    
    <div class="range">
    
      <xsl:if test="$percentage &gt; 5">
        <div class="level">
          <xsl:attribute name="style">
            <xsl:value-of 
              select="concat('width: ', $percentage, 'px;')"/>
          </xsl:attribute>
        </div>
      </xsl:if>
    
    </div>
    
    <span>
      <xsl:value-of select="concat(round($percentage), $units)"/>
    </span>
    
  </xsl:template>

  <!-- Top-level wrapper that creates the section container and title. -->
  <xsl:template match="*" mode="top">

    <!-- Grab the name of the current node and its transformed content. -->
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name" select="local-name()"/>
      <xsl:with-param name="contents">
        <xsl:apply-templates select="."/>
      </xsl:with-param>
    </xsl:call-template>
    
  </xsl:template>
 
  <!-- Top-level wrapper that creates the section container and title. -->
  <xsl:template name="section-wrapper">
    <xsl:param name="name"/>
    <xsl:param name="control"/>
    <xsl:param name="contents"/>

    <!-- Get the section title. -->
    <xsl:variable 
      name="title" select="$strings/sections/value[@key = $name]"/>
    
    <!-- Construct a wrapper div with class values of sectino and $name. -->
    <div>
      <xsl:attribute name="id">
        <xsl:value-of select="$name"/>
      </xsl:attribute>
      
      <xsl:attribute name="class">
        <xsl:value-of select="concat('section ', $name)"/>
      </xsl:attribute>
      
      <!-- Add a title if there is one. -->
      <xsl:if test="$title">
        <xsl:copy-of select="$control"/>
        <h1>
          <xsl:value-of select="$title"/>
        </h1>
      </xsl:if>
      
      <!-- Insert the contents. -->
      <xsl:copy-of select="$contents"/>
      
    </div>
    
  </xsl:template>

  <xsl:include href="Library.xsl" />

</xsl:stylesheet>
